package com.diagbot.aggregate;

import com.diagbot.client.PushNewServiceClient;
import com.diagbot.dto.PushBaseDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.PushNewDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.DiagnoseTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.process.PushProcess;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.PushNewVO;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 提醒类聚合
 * @author: zhoutg
 * @time: 2019/10/16 13:37
 */
@Component
@Slf4j
public class PushNewAggregate {

    @Autowired
    PushProcess pushProcess;
    @Autowired
    PushNewServiceClient pushNewServiceClient;

    @DataProvider("pushAll")
    public PushDTO push(
            @InvokeParameter("wordCrfDTO") WordCrfDTO wordCrfDTO,
            @InvokeParameter("pushNewVO") PushNewVO pushNewVO,
            @DataConsumer("pushBigData") PushDTO pushBigData,
            @DataConsumer("pushDiagnose") PushDTO pushDiagnose) {

        PushDTO res = new PushDTO();
        Map<String, Object> debug = new LinkedHashMap<>();
        Map<String, List<PushBaseDTO>> dis = new LinkedHashMap<>();
        res.setDis(dis);
        res.setDebug(debug);
        if (pushDiagnose != null) {
            if (pushDiagnose.getDis() != null) {
                dis.putAll(pushDiagnose.getDis());
            }
            debug.putAll(pushDiagnose.getDebug());
        }
        if (pushBigData != null) {
            if (pushBigData.getDis() != null) {
                dis.putAll(pushBigData.getDis());
            }
            debug.putAll(pushBigData.getDebug());
        }
        return res;
    }

    /**
     * 诊断依据推理
     *
     * @param wordCrfDTO
     * @return
     */
    @DataProvider("pushDiagnose")
    public PushDTO pushDiagnose(@InvokeParameter("wordCrfDTO") WordCrfDTO wordCrfDTO) {
        PushDTO pushDTO = pushProcess.pushDiagnose(wordCrfDTO);
        return pushDTO;
    }

    /**
     * 大数据推理
     *
     * @param pushNewVO
     * @return
     */
    @DataProvider("pushBigData")
    public PushDTO pushBigData(@InvokeParameter("pushNewVO") PushNewVO pushNewVO,
                               @InvokeParameter("wordCrfDTO") WordCrfDTO wordCrfDTO) {
        PushDTO pushDTO = new PushDTO();
        Map<String, List<PushBaseDTO>> dis = new LinkedHashMap<>();
        pushDTO.setDis(dis);
        long start = System.currentTimeMillis();
        pushNewVO.setWordCrfDTO(wordCrfDTO);
        try {
            PushNewDTO pushNewDTO = pushNewServiceClient.pushNew(pushNewVO);
            if (pushNewDTO == null || (pushNewDTO != null && pushNewDTO.getStatus().equals(-1))) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "大数据推理出错");
            }
            // 根据现有信息，无法推送相关疾病
            if (pushNewDTO.getStatus().equals(1)) {
                CoreUtil.getDebugStr("【大数据推理无诊断结果】", "无法推送相关疾病", pushDTO.getDebug());
            } else if (pushNewDTO.getStatus().equals(0)) {
                if (ListUtil.isNotEmpty(pushNewDTO.getDisease())) {
                    List<PushBaseDTO> pushBaseDTOList = pushNewDTO.getDisease().stream().map(r -> {
                        PushBaseDTO pushBaseDTO = new PushBaseDTO();
                        pushBaseDTO.setName(r);
                        return pushBaseDTO;
                    }).collect(Collectors.toList());
                    dis.put(DiagnoseTypeEnum.bigDataPush.getName(), pushBaseDTOList);
                }
            }
        } catch (Exception e) {
            log.error("【大数据推理出错】", e);
            CoreUtil.getDebugStr("【大数据推理出错】", "大数据推理出错", pushDTO.getDebug());
        }
        CoreUtil.getDebugStr(start, "推送诊断耗时_大数据", pushDTO.getDebug());
        return pushDTO;
    }
}
