package com.diagbot.config;

import com.diagbot.facade.CacheFacade;
import com.diagbot.facade.ConceptInfoFacade;
import com.diagbot.facade.NeoFacade;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

/**
 * @Description: 项目启动后初始化缓存
 * @author: gaodm
 * @time: 2020/5/13 11:20
 */
@Component //把类交给spring容器管理
@Order(100)  //使用order属性，设置该类在spring容器中的加载顺序
@Slf4j
public class CacheDeleteInit implements CommandLineRunner {

    @Autowired
    CacheFacade cacheFacade;
    @Autowired
    NeoFacade neoFacade;
    @Autowired
    private ConceptInfoFacade conceptInfoFacade;


    @Override
    public void run(String... args) {
        // 服务启动清除redis缓存
        cacheFacade.clear();
        log.info("CDSS-CORE服务启动清除redis缓存成功！");

        cacheFacade.loadDrugTypeCache();
        log.info("CDSS-CORE服务启动加载药品类型对应关系缓存成功！");

        cacheFacade.getDiseaseCorrespondCache();
        log.info("CDSS-CORE服务启动加载疾病对应ICD10缓存成功！");

        cacheFacade.getdiseaseFilterCache();
        log.info("CDSS-CORE服务启动加载疾病过滤缓存成功！");

        cacheFacade.loadPushModel();
        log.info("CDSS-CORE服务启动加载医院推送类型成功！");

        cacheFacade.loadDeptPush();
        log.info("CDSS-CORE服务启动加载科室和推送映射成功！");

        cacheFacade.loadAllRuleCache();
        log.info("CDSS-CORE服务启动加载医学知识库中的所有规则缓存成功！");

        cacheFacade.loadAllBaseDiagnoseCache();
        log.info("CDSS-CORE服务启动加载诊断依据缓存成功！");

        conceptInfoFacade.loadCustomDictionary();
        log.info("CDSS-CORE服务启动加载NLP分词字典成功！");

        log.info("CDSS-CORE初始化完成！");
    }
}
