package com.diagbot.entity;

import cn.afterturn.easypoi.excel.annotation.Excel;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;

import javax.validation.constraints.NotBlank;
import java.io.Serializable;
import java.util.Date;
import java.util.Objects;

/**
 * <p>
 * 药品映射表
 * </p>
 *
 * @author zhaops
 * @since 2020-07-28
 */
@TableName("tran_drug_config")
public class DrugConfig implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 是否删除,N:未删除，Y:删除
     */
    private String isDeleted;

    /**
     * 记录创建时间
     */
    private Date gmtCreate;

    /**
     * 记录修改时间,如果时间是1970年则表示纪录未修改
     */
    private Date gmtModified;

    /**
     * 创建人，0表示无创建人值
     */
    private String creator;

    /**
     * 修改人,如果为0则表示纪录未修改
     */
    private String modifier;

    /**
     * 医院id
     */
    private Long hospitalId;

    /**
     * 医院药品名称
     */
    @Excel(name = "医院药品名称", width = 40, orderNum = "1", isImportField = "true")
    @NotBlank(message = "请输入医院药品名称")
    //@Size(max = 80, min = 1,message = "医院药品名称长度需要在1-80字符长度之间")
    private String hisName;

    /**
     * 标准药品名称
     */
    @Excel(name = "标准药品名称", width = 40, orderNum = "2", isImportField = "true")
    @NotBlank(message = "请输入标准药品名称")
    //@Size(max = 80, min = 1,message = "标准药品名称长度需要在1-80字符长度之间")
    private String uniqueName;

    /**
     * 标准编码
     */
    //@Excel(name = "对应项编码", width = 40, orderNum = "3")
    private String uniqueCode;

    /**
     * 剂型
     */
    @Excel(name = "药品剂型", width = 60, orderNum = "3", isImportField = "true")
    private String form;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getIsDeleted() {
        return isDeleted;
    }

    public void setIsDeleted(String isDeleted) {
        this.isDeleted = isDeleted;
    }

    public Date getGmtCreate() {
        return gmtCreate;
    }

    public void setGmtCreate(Date gmtCreate) {
        this.gmtCreate = gmtCreate;
    }

    public Date getGmtModified() {
        return gmtModified;
    }

    public void setGmtModified(Date gmtModified) {
        this.gmtModified = gmtModified;
    }

    public String getCreator() {
        return creator;
    }

    public void setCreator(String creator) {
        this.creator = creator;
    }

    public String getModifier() {
        return modifier;
    }

    public void setModifier(String modifier) {
        this.modifier = modifier;
    }

    public Long getHospitalId() {
        return hospitalId;
    }

    public void setHospitalId(Long hospitalId) {
        this.hospitalId = hospitalId;
    }

    public String getHisName() {
        return hisName;
    }

    public void setHisName(String hisName) {
        this.hisName = hisName;
    }

    public String getUniqueName() {
        return uniqueName;
    }

    public void setUniqueName(String uniqueName) {
        this.uniqueName = uniqueName;
    }

    public String getUniqueCode() {
        return uniqueCode;
    }

    public void setUniqueCode(String uniqueCode) {
        this.uniqueCode = uniqueCode;
    }

    public String getForm() {
        return form;
    }

    public void setForm(String form) {
        this.form = form;
    }

    @Override
    public String toString() {
        return "DrugConfig{" +
                "id=" + id +
                ", isDeleted=" + isDeleted +
                ", gmtCreate=" + gmtCreate +
                ", gmtModified=" + gmtModified +
                ", creator=" + creator +
                ", modifier=" + modifier +
                ", hospitalId=" + hospitalId +
                ", hisName=" + hisName +
                ", uniqueName=" + uniqueName +
                ", uniqueCode=" + uniqueCode +
                ", form=" + form +
                "}";
    }

    @Override
    public boolean equals(Object o) {
        if (this == o)
            return true;
        if (o == null || getClass() != o.getClass())
            return false;
        DrugConfig drugConfig = (DrugConfig) o;
        return Objects.equals(id, drugConfig.id)
                && Objects.equals(isDeleted, drugConfig.isDeleted)
                && Objects.equals(hospitalId, drugConfig.hospitalId)
                && Objects.equals(hisName, drugConfig.hisName)
                && Objects.equals(form, drugConfig.form)
                && Objects.equals(uniqueName, drugConfig.uniqueName)
                && Objects.equals(uniqueCode, drugConfig.uniqueCode);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, isDeleted, hospitalId, hisName, form, uniqueName, uniqueCode);
    }
}