package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.dto.CustomDictionaryDTO;
import com.diagbot.dto.ReverseDTO;
import com.diagbot.entity.ConceptInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.ConceptInfoServiceImpl;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.ReverseVO;
import com.google.common.collect.Lists;
import com.hankcs.hanlp.dictionary.CustomDictionary;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/12/28 11:47
 */
@Component
public class ConceptInfoFacade extends ConceptInfoServiceImpl {

    /**
     * 标准术语名称修改
     *
     * @param oldName
     * @param newName
     * @return
     */
    public Boolean updateName(String oldName, String newName, String labelType) {
        UpdateWrapper<ConceptInfo> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("name", oldName)
                .eq("type", labelType)
                .set("name", newName);
        this.update(updateWrapper);
        return true;
    }

    /**
     * 根据诊断反推
     *
     * @param reverseVO
     * @return
     */
    public List<ReverseDTO> getReverseFac(ReverseVO reverseVO) {
        if (reverseVO != null && ListUtil.isNotEmpty(reverseVO.getRelationType())) {
            return this.getReverse(reverseVO);
        }
        return Lists.newArrayList();
    }

    public void loadCustomDictionary(){
        CustomDictionary.reload();
        List<CustomDictionaryDTO> words = this.getAllWord();
        if (ListUtil.isNotEmpty(words)){
            for (CustomDictionaryDTO dictionaryDTO: words){
                CustomDictionary.insert(dictionaryDTO.getName().trim().replaceAll(" ",""), dictionaryDTO.getType());
            }
        }
    }
}