package com.diagbot.facade;

import com.diagbot.client.StandConvertServiceClient;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ParamUtil;
import com.diagbot.vo.IndicationPushVO;
import io.github.lvyahui8.spring.facade.DataFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 提示信息facade
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class IndicationFacade {

    @Autowired
    NeoFacade neoFacade;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    BillFacade billFacade;
    @Autowired
    StandConvertServiceClient standConvertServiceClient;
    @Autowired
    KlRegularConfigFacade klRegularConfigFacade;

    // private static final Map<String, List<String>> methodMap;
    // static {
    //     methodMap = new HashMap<>();
    //     methodMap.put("1", Arrays.asList("crf", "stand"));
    //     methodMap.put("2", Arrays.asList("crf", "stand"));
    //     methodMap.put("3", Arrays.asList("crf", "stand"));
    //     methodMap.put("4", Arrays.asList("crf", "stand"));
    // }

    /**
     * 提示总入口
     *
     * @param
     * @return
     */
    public IndicationDTO indicationFac(IndicationPushVO indicationPushVO) {
        IndicationDTO res = new IndicationDTO();
        Map<String, Object> debug = new LinkedHashMap<>();
        //方法适配处理
        List<String> ruleTypeList = Arrays.asList(indicationPushVO.getRuleType().split(","));
        indicationPushVO.setRuleTypeList(ruleTypeList);

        long start = System.currentTimeMillis();
        // CRF模型处理
        WordCrfDTO wordCrfDTO = commonFacade.crf_process(indicationPushVO, false);
        CoreUtil.getDebugStr(start, "模型处理耗时", debug);

        // 同义词转换
        long standStart = System.currentTimeMillis();
        commonFacade.wordStandConvert(wordCrfDTO);
        CoreUtil.getDebugStr(standStart, "标准词转换耗时", debug);

        //正则匹配开始
        wordCrfDTO = klRegularConfigFacade.getRegularConfigs(wordCrfDTO, indicationPushVO);
        //正则匹配结束
        // 合并【数据来源】
        wordCrfDTO.setDiagSource(commonFacade.getDiseaseSource(wordCrfDTO));
        wordCrfDTO.setDrugSource(commonFacade.getDrugSource(wordCrfDTO));
        // 化验结果特殊值处理
        ParamUtil.dealLisIndication(wordCrfDTO.getLis());

        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("wordCrfDTO", wordCrfDTO);
            invokeParams.put("indicationPushVO", indicationPushVO);
            invokeParams.put("debug", debug);
            res = DataFacade.get("indicationAll", invokeParams, IndicationDTO.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "提醒类服务出错" + e.getMessage());
        }

        // 输入调试信息
        CoreUtil.getDebugStr(start, "本次调用总计耗时", res.getDebug());
        CoreUtil.getDebugObject("数据", res.getDebug(), wordCrfDTO);
        return res;
    }

    /**
     * 提示总入口
     *
     * @param
     * @return
     */
    public WordCrfDTO getWordCrfDTO(IndicationPushVO indicationPushVO) {
        // CRF模型处理
        WordCrfDTO wordCrfDTO = commonFacade.crf_process(indicationPushVO, true);
        // 同义词转换
        commonFacade.wordStandConvert(wordCrfDTO);
        return wordCrfDTO;
    }
}
