package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.GetAllForRelationDTO;
import com.diagbot.dto.IndexBatchDTO;
import com.diagbot.dto.KllisDetailDTO;
import com.diagbot.entity.KlConcept;
import com.diagbot.entity.KlDisease;
import com.diagbot.entity.KlLibraryInfo;
import com.diagbot.entity.KlOperation;
import com.diagbot.entity.TcmDisease;
import com.diagbot.entity.TcmSyndrome;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.MatchSourceEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.service.impl.KlConceptServiceImpl;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.KllisDetailVO;
import com.diagbot.vo.SearchConceptVO;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @author: gaodm
 * @time: 2021/1/26 13:05
 */
@Component
public class KlConceptFacade extends KlConceptServiceImpl {

    @Autowired
    private KlDiseaseFacade klDiseaseFacade;
    @Autowired
    private KlOperationFacade klOperationFacade;
    @Autowired
    private TcmDiseaseFacade tcmDiseaseFacade;
    @Autowired
    private TcmSyndromeFacade tcmSyndromeFacade;
    @Autowired
    private KlLibraryInfoFacade klLibraryInfoFacade;

    /**
     * 批量校验标准术语
     *
     * @param conceptVO
     * @return
     */
    public List<IndexBatchDTO> getConceptNames(ConceptVO conceptVO) {
        if (ListUtil.isEmpty(conceptVO.getNames()) && ListUtil.isEmpty(conceptVO.getIds())) {
            return new ArrayList<>();
        }
        List<IndexBatchDTO> retList = Lists.newLinkedList();
        //type： 类型：1-化验大项、2-化验小项、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候
        Integer type = convertType(conceptVO.getType(), false);

        if (conceptVO.getSource().equals(-1)) {
            //根据id检索
            List<KlConcept> concepts = this.list(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("status", StatusEnum.Enable.getKey())
                    .in("id", conceptVO.getIds()));
            for (KlConcept concept : concepts) {
                IndexBatchDTO dto = new IndexBatchDTO();
                dto.setId(concept.getId());
                dto.setName(concept.getLibName());
                dto.setType(convertType(concept.getLibType(), true));
                retList.add(dto);
            }
        } else if (conceptVO.getSource().equals(MatchSourceEnum.StandWord.getKey())) {
            QueryWrapper<KlConcept> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("status", StatusEnum.Enable.getKey())
                    .in("lib_name", conceptVO.getNames());
            if (conceptVO.getType().equals(ConceptTypeEnum.Pacs.getKey())) {
                //辅检同时检索辅检项目和辅检子项
                queryWrapper.in("lib_type", Arrays.asList(LexiconEnum.PacsName.getKey(), LexiconEnum.PacsSubName.getKey()));
            } else {
                queryWrapper.eq("lib_type", type);
            }
            List<KlConcept> concepts = this.list(queryWrapper);

            if (ListUtil.isNotEmpty(concepts)) {
                for (KlConcept concept : concepts) {
                    IndexBatchDTO dto = new IndexBatchDTO();
                    dto.setId(concept.getId());
                    dto.setName(concept.getLibName());
                    retList.add(dto);
                }
            }
        } else if (conceptVO.getSource().equals(MatchSourceEnum.SynonymsWord.getKey())) {
            QueryWrapper<KlLibraryInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("is_concept", 0)
                    .in("name", conceptVO.getNames());
            if (conceptVO.getType().equals(ConceptTypeEnum.Pacs.getKey())) {
                //辅检同时检索辅检项目和辅检子项
                queryWrapper.in("type_id", Arrays.asList(LexiconEnum.PacsName.getKey(), LexiconEnum.PacsSubName.getKey()));
            } else {
                queryWrapper.eq("type_id", type);
            }
            List<KlLibraryInfo> libratyInfos = klLibraryInfoFacade.list(queryWrapper);
            if (ListUtil.isEmpty(libratyInfos)) {
                return retList;
            }
            Map<Long, List<KlLibraryInfo>> libraryInfoMap
                    = libratyInfos.stream().collect(Collectors.groupingBy(KlLibraryInfo::getConceptId));
            List<Long> synonymsRelatedConceptIds
                    = libratyInfos.stream().map(KlLibraryInfo::getConceptId).collect(Collectors.toList());
            List<KlConcept> synonymsRelatedConcepts = this.list(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("status", StatusEnum.Enable.getKey())
                    .in("id", synonymsRelatedConceptIds));
            Map<Long, KlConcept> conceptMap
                    = synonymsRelatedConcepts.stream().collect(Collectors.toMap(KlConcept::getId, v -> v));

            if (conceptMap != null && conceptMap.size() > 0) {
                for (Map.Entry<Long, KlConcept> entry : conceptMap.entrySet()) {
                    KlConcept concept = entry.getValue();
                    if (libraryInfoMap.containsKey(entry.getKey())) {
                        for (KlLibraryInfo libraryInfo : libraryInfoMap.get(entry.getKey())) {
                            if (!conceptVO.getNames().contains(libraryInfo.getName())) {
                                continue;
                            }
                            IndexBatchDTO indexBatchDTO = new IndexBatchDTO();
                            indexBatchDTO.setId(concept.getId());
                            indexBatchDTO.setName(concept.getLibName());
                            indexBatchDTO.setSynonyms(libraryInfo.getName());
                            retList.add(indexBatchDTO);
                        }
                    }
                }
            }
        } else if (conceptVO.getSource().equals(MatchSourceEnum.Code.getKey())) {
            List<Long> conceptIds = Lists.newArrayList();
            Map<Long, List<String>> codeMap = new HashMap<>();
            if (type.equals(LexiconEnum.Disease.getKey())) {
                List<KlDisease> diseases = klDiseaseFacade.list(new QueryWrapper<KlDisease>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .in("icd10_code", conceptVO.getNames()));
                if (ListUtil.isNotEmpty(diseases)) {
                    conceptIds = diseases.stream().map(KlDisease::getConceptId).distinct().collect(Collectors.toList());
                    codeMap = diseases.stream().collect(Collectors.toMap(KlDisease::getConceptId,
                            v -> {
                                List<String> codes = Lists.newArrayList();
                                codes.add(v.getIcd10Code());
                                return codes;
                            }));
                }
            } else if (type.equals(LexiconEnum.Operation.getKey())) {
                List<KlOperation> operations = klOperationFacade.list(new QueryWrapper<KlOperation>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .in("operation_code", conceptVO.getNames()));
                if (ListUtil.isNotEmpty(operations)) {
                    conceptIds = operations.stream().map(KlOperation::getConceptId).distinct().collect(Collectors.toList());
                    codeMap = operations.stream().collect(Collectors.toMap(KlOperation::getConceptId,
                            v -> {
                                List<String> codes = Lists.newArrayList();
                                codes.add(v.getOperationCode());
                                return codes;
                            }));
                }
            } else if (type.equals(LexiconEnum.Tcmdisease.getKey())) {
                List<TcmDisease> tcmDiseases = tcmDiseaseFacade.list(new QueryWrapper<TcmDisease>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .in("code", conceptVO.getNames()));
                if (ListUtil.isNotEmpty(tcmDiseases)) {
                    conceptIds = tcmDiseases.stream().map(TcmDisease::getConceptId).distinct().collect(Collectors.toList());
                    codeMap = tcmDiseases.stream().collect(Collectors.toMap(TcmDisease::getConceptId,
                            v -> {
                                List<String> codes = Lists.newArrayList();
                                codes.add(v.getCode());
                                return codes;
                            }));
                }
            } else if (type.equals(LexiconEnum.Tcmsyndrome.getKey())) {
                List<TcmSyndrome> tcmSyndromes = tcmSyndromeFacade.list(new QueryWrapper<TcmSyndrome>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .in("code", conceptVO.getNames()));
                if (ListUtil.isNotEmpty(tcmSyndromes)) {
                    conceptIds = tcmSyndromes.stream().map(TcmSyndrome::getConceptId).distinct().collect(Collectors.toList());
                    codeMap = tcmSyndromes.stream().collect(Collectors.toMap(TcmSyndrome::getConceptId,
                            v -> {
                                List<String> codes = Lists.newArrayList();
                                codes.add(v.getCode());
                                return codes;
                            }));
                }
            }
            List<KlConcept> concepts = this.list(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("status", StatusEnum.Enable.getKey())
                    .in("id", conceptIds));
            Map<Long, KlConcept> conceptMap
                    = concepts.stream().collect(Collectors.toMap(KlConcept::getId, v -> v));
            if (conceptMap != null && conceptMap.size() > 0) {
                for (Map.Entry<Long, KlConcept> entry : conceptMap.entrySet()) {
                    KlConcept concept = entry.getValue();
                    if (codeMap.containsKey(entry.getKey())) {
                        for (String code : codeMap.get(entry.getKey())) {
                            if (!conceptVO.getNames().contains(code)) {
                                continue;
                            }
                            IndexBatchDTO indexBatchDTO = new IndexBatchDTO();
                            indexBatchDTO.setId(concept.getId());
                            indexBatchDTO.setName(concept.getLibName());
                            indexBatchDTO.setCode(code);
                            retList.add(indexBatchDTO);
                        }
                    }
                }
            }
        }

        if (ListUtil.isEmpty(retList)) {
            return retList;
        }
        //关联编码
        List<Long> conceptIds = retList.stream().map(IndexBatchDTO::getId).collect(Collectors.toList());
        if (type.equals(LexiconEnum.Disease.getKey())) {
            List<KlDisease> diseases = klDiseaseFacade.list(new QueryWrapper<KlDisease>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("concept_id", conceptIds));
            if (ListUtil.isNotEmpty(diseases)) {
                Map<Long, KlDisease> idMap = diseases.stream().collect(Collectors.toMap(KlDisease::getConceptId, v -> v));
                for (IndexBatchDTO dto : retList) {
                    if (idMap.containsKey(dto.getId())) {
                        dto.setCode(idMap.get(dto.getId()).getIcd10Code());
                    }
                }
            }
        } else if (type.equals(LexiconEnum.Operation.getKey())) {
            List<KlOperation> operations = klOperationFacade.list(new QueryWrapper<KlOperation>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("concept_id", conceptIds));
            if (ListUtil.isNotEmpty(operations)) {
                Map<Long, KlOperation> idMap = operations.stream().collect(Collectors.toMap(KlOperation::getConceptId, v -> v));
                for (IndexBatchDTO dto : retList) {
                    if (idMap.containsKey(dto.getId())) {
                        dto.setCode(idMap.get(dto.getId()).getOperationCode());
                    }
                }
            }
        } else if (type.equals(LexiconEnum.Tcmdisease.getKey())) {
            List<TcmDisease> tcmDiseases = tcmDiseaseFacade.list(new QueryWrapper<TcmDisease>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("concept_id", conceptIds));
            if (ListUtil.isNotEmpty(tcmDiseases)) {
                Map<Long, TcmDisease> idMap = tcmDiseases.stream().collect(Collectors.toMap(TcmDisease::getConceptId, v -> v));
                for (IndexBatchDTO dto : retList) {
                    if (idMap.containsKey(dto.getId())) {
                        dto.setCode(idMap.get(dto.getId()).getCode());
                    }
                }
            }
        } else if (type.equals(LexiconEnum.Tcmsyndrome.getKey())) {
            List<TcmSyndrome> tcmSyndromes = tcmSyndromeFacade.list(new QueryWrapper<TcmSyndrome>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("concept_id", conceptIds));
            if (ListUtil.isNotEmpty(tcmSyndromes)) {
                Map<Long, TcmSyndrome> idMap = tcmSyndromes.stream().collect(Collectors.toMap(TcmSyndrome::getConceptId, v -> v));
                for (IndexBatchDTO dto : retList) {
                    if (idMap.containsKey(dto.getId())) {
                        dto.setCode(idMap.get(dto.getId()).getCode());
                    }
                }
            }
        }

        return retList;
    }


    public Integer convertType(Integer type, Boolean reverse) {
        Integer retType = null;
        //type： 类型：1-化验大项、2-化验小项、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候
        if (reverse) {
            if (type.equals(LexiconEnum.LisName.getKey())
                    || type.equals(LexiconEnum.LisSubName.getKey())) {
                retType = ConceptTypeEnum.LisPack.getKey();
            } else if (type.equals(LexiconEnum.PacsName.getKey())
                    || type.equals(LexiconEnum.PacsSubName.getKey())) {
                retType = ConceptTypeEnum.Pacs.getKey();
            } else if (type.equals(LexiconEnum.Disease.getKey())) {
                retType = ConceptTypeEnum.Disease.getKey();
            } else if (type.equals(LexiconEnum.Medicine.getKey())) {
                retType = ConceptTypeEnum.Drug.getKey();
            } else if (type.equals(LexiconEnum.Operation.getKey())) {
                retType = ConceptTypeEnum.Operation.getKey();
            } else if (type.equals(LexiconEnum.Dept.getKey())) {
                retType = ConceptTypeEnum.Dept.getKey();
            } else if (type.equals(LexiconEnum.Transfusion.getKey())) {
                retType = ConceptTypeEnum.Transfusion.getKey();
            } else if (type.equals(LexiconEnum.Scale.getKey())) {
                retType = ConceptTypeEnum.Scale.getKey();
            } else if (type.equals(LexiconEnum.Nurse.getKey())) {
                retType = ConceptTypeEnum.Nurse.getKey();
            } else if (type.equals(LexiconEnum.Tcmdisease.getKey())) {
                retType = ConceptTypeEnum.Tcmdisease.getKey();
            } else if (type.equals(LexiconEnum.Tcmsyndrome.getKey())) {
                retType = ConceptTypeEnum.Tcmsyndrome.getKey();
            }
        } else {
            switch (type) {
                case 1:
                    retType = LexiconEnum.LisName.getKey();
                    break;
                case 2:
                    retType = LexiconEnum.LisSubName.getKey();
                    break;
                case 3:
                    retType = LexiconEnum.PacsName.getKey();
                    break;
                case 4:
                    retType = LexiconEnum.Disease.getKey();
                    break;
                case 5:
                    retType = LexiconEnum.Medicine.getKey();
                    break;
                case 6:
                    retType = LexiconEnum.Operation.getKey();
                    break;
                case 7:
                    retType = LexiconEnum.Dept.getKey();
                    break;
                case 8:
                    retType = LexiconEnum.Transfusion.getKey();
                    break;
                case 10:
                    retType = LexiconEnum.Scale.getKey();
                    break;
                case 11:
                    retType = LexiconEnum.Nurse.getKey();
                    break;
                case 12:
                    retType = LexiconEnum.Tcmdisease.getKey();
                    break;
                case 13:
                    retType = LexiconEnum.Tcmsyndrome.getKey();
                    break;
                default:
                    break;
            }
        }
        return retType;
    }

    public List<GetAllForRelationDTO> searchConceptByNameAndLibType(SearchConceptVO searchConceptVO) {
        List<GetAllForRelationDTO> getAllForRelationDTOS = Lists.newArrayList();
        String name = searchConceptVO.getName();
        Integer libType = searchConceptVO.getLibType();
        List<Long> excludedConceptIds = searchConceptVO.getExcludedConceptIds();
        if (StringUtils.isNotBlank(name) && libType != null) {
            List<KlConcept> conceptList = this.list(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .like(StringUtil.isNotBlank(name), "lib_name", name.trim())
                    .eq("lib_type", libType)
                    .eq("status", StatusEnum.Enable.getKey())
                    .notIn(ListUtil.isNotEmpty(excludedConceptIds), "id", excludedConceptIds));
            if (ListUtil.isNotEmpty(conceptList)) {
                getAllForRelationDTOS = conceptList.stream().map(x -> {
                    GetAllForRelationDTO getAllForRelationDTO = new GetAllForRelationDTO();
                    getAllForRelationDTO.setConceptNameType(x.getLibName());
                    getAllForRelationDTO.setConceptName(x.getLibName());
                    getAllForRelationDTO.setConceptId(x.getId());
                    getAllForRelationDTO.setLibType(x.getLibType());
                    return getAllForRelationDTO;
                }).collect(Collectors.toList());
            }
        }
        return getAllForRelationDTOS;
    }

    public Map<String, KllisDetailDTO> getLisDetaisByNamesFac(KllisDetailVO kllisDetailVO) {
        List<KllisDetailDTO> list = this.getLisDetaisByNames(kllisDetailVO);
        Map<String, KllisDetailDTO> res = EntityUtil.makeEntityMap(list, "libName");
        return res;
    }
}
