package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.entity.LisConfig;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.LisConfigServiceImpl;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/7/29 15:03
 */
@Component
public class LisConfigFacade extends LisConfigServiceImpl {

    /**
     * 获取映射关系-id
     * Map<hisName,Map<hisDetailName,Map<uniqueName,id>>>
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Map<String, Long>>> getConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Map<String, Long>>> retMap = new HashMap<>();
        QueryWrapper<LisConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<LisConfig> records = this.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }

        records.forEach(lisConfig -> {
            if (lisConfig.getHisDetailName() == null) {
                lisConfig.setHisDetailName("");
            }
        });

        Map<String, List<LisConfig>> hisNameMap = EntityUtil.makeEntityListMap(records, "hisName");
        for (Map.Entry<String, List<LisConfig>> entry : hisNameMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                Map<String, Map<String, Long>> detailNameMap = new HashMap<>();
                //his名称映射到标准词，1：n
                Map<String, List<LisConfig>> hisDetailNameMap
                        = EntityUtil.makeEntityListMap(entry.getValue(), "hisDetailName");
                for (Map.Entry<String, List<LisConfig>> detailEntry : hisDetailNameMap.entrySet()) {
                    if (ListUtil.isNotEmpty(detailEntry.getValue())) {
                        detailNameMap.put(detailEntry.getKey(),
                                EntityUtil.makeMapWithKeyValue(detailEntry.getValue(), "uniqueName", "id"));
                    }
                }
                retMap.put(entry.getKey(), detailNameMap);
            }
        }
        return retMap;
    }

    /**
     * 已映射
     *
     * @param name
     * @return
     */
    public Boolean isMapping(String name) {
        QueryWrapper<LisConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("unique_name", name);
        List<LisConfig> records = this.list(queryWrapper);
        if (ListUtil.isNotEmpty(records)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 标准术语名称修改
     *
     * @param oldName
     * @param newName
     * @return
     */
    public Boolean updateUniqueName(String oldName, String newName) {
        UpdateWrapper<LisConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("unique_name", oldName)
                .set("unique_name", newName);
        this.update(updateWrapper);
        return true;
    }

    /**
     * 获取过滤化验套餐名= 映射套餐名称 + 结构化套餐名称
     *
     * @param wordCrfDTO
     * @return
     */
    public List<String> getMappingLisItem(WordCrfDTO wordCrfDTO) {
        if (ListUtil.isNotEmpty(wordCrfDTO.getLis())) {
            // 医院化验套餐名
            List<String> itemList = wordCrfDTO.getLis().stream().map(r -> r.getName()).collect(Collectors.toList());
            // 获取标准套餐名
            List<LisConfig> records = this.list(new QueryWrapper<LisConfig>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("hospital_id", wordCrfDTO.getHospitalId())
                    .in("his_name", itemList)
                    .apply("(his_detail_name is null or his_detail_name = '')")
            );
            // 获取映射套餐名称
            List<String> uniqueName = records.stream().map(r -> r.getUniqueName()).collect(Collectors.toList());
            // 追加医院套餐名称
            uniqueName.addAll(itemList);
            // 去重
            uniqueName = CoreUtil.removeRepeat(uniqueName);
            return uniqueName;
        }
        return Lists.newArrayList();
    }
}