package com.diagbot.facade;

import com.google.common.collect.Lists;
import com.hankcs.hanlp.HanLP;
import com.hankcs.hanlp.seg.common.Term;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @author: gaodm
 * @time: 2021/5/25 20:13
 */
@Component
public class NLPFacade {

    public String segment(String text) {
        return HanLP.segment(text).toString();
    }

    /**
     * 返回所有结果
     *
     * @param text
     * @return
     */
    public Map<String, List<String>> getMap(String text) {
        Map<String, List<String>> map = new LinkedHashMap<>();
        List<Term> termList = HanLP.segment(text);
        for (Term term : termList) {
            String word = term.word;
            String type = term.nature.toString();
            List<String> splitTypeList = Lists.newArrayList(type.split(","));
            for (String t : splitTypeList) {
                if (map.get(t) != null) {
                    map.get(t).add(word);
                } else {
                    List<String> wordList = Lists.newArrayList();
                    wordList.add(word);
                    map.put(t, wordList);
                }
            }
        }
        return map;
    }

    /**
     * 根据指定指定返回结果
     *
     * @param text
     * @param filterType
     * @return
     */
    public List<String> getByType(String text, String filterType) {
        List<String> res = Lists.newArrayList();
        List<Term> termList = HanLP.segment(text);
        for (Term term : termList) {
            String word = term.word;
            String type = term.nature.toString();
            List<String> splitTypeList = Lists.newArrayList(type.split(","));
            for (String splitType : splitTypeList) {
                if (filterType.equals(splitType)) {
                    res.add(word);
                }
            }
        }
        return res;
    }

}