package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.entity.PacsConfig;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.PacsConfigServiceImpl;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/7/29 15:03
 */
@Component
public class PacsConfigFacade extends PacsConfigServiceImpl {

    /**
     * 获取映射关系-公表名
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Long>> getConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<PacsConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<PacsConfig> records = this.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        Map<String, List<PacsConfig>> configMap = EntityUtil.makeEntityListMap(records, "hisName");
        for (Map.Entry<String, List<PacsConfig>> entry : configMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(), EntityUtil.makeMapWithKeyValue(entry.getValue(), "uniqueName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 已映射
     *
     * @param name
     * @return
     */
    public Boolean isMapping(String name) {
        QueryWrapper<PacsConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("unique_name", name);
        List<PacsConfig> records = this.list(queryWrapper);
        if (ListUtil.isNotEmpty(records)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 标准术语名称修改
     *
     * @param oldName
     * @param newName
     * @return
     */
    public Boolean updateUniqueName(String oldName, String newName) {
        UpdateWrapper<PacsConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("unique_name", oldName)
                .set("unique_name", newName);
        this.update(updateWrapper);
        return true;
    }
}