package com.diagbot.facade;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.client.PushNewServiceClient;
import com.diagbot.dto.PushBaseDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.PushNewDTO;
import com.diagbot.dto.PushPlanDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.DiagnoseTypeEnum;
import com.diagbot.enums.RedisEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.model.entity.Diag;
import com.diagbot.process.PushProcess;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.ParamUtil;
import com.diagbot.util.RedisUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.PushNewVO;
import com.diagbot.vo.PushPlanVO;
import com.diagbot.vo.PushVO;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.facade.DataFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 推送facade
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class PushFacade {

    @Autowired
    PushProcess pushProcess;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    NeoFacade neoFacade;
    @Autowired
    FollowupPlanInfoFacade followupPlanInfoFacade;
    @Autowired
    TranHospitalInfoFacade tranHospitalInfoFacade;
    @Autowired
    PushNewServiceClient pushNewServiceClient;
    @Autowired
    RedisUtil redisUtil;
    @Value("${PushNew.url}")
    String pushUrl;

    /**
     * 推送fac(兼容新版和5.0)
     *
     * @param pushVO
     * @return
     */
    public PushDTO pushFac(PushVO pushVO) {
        // 获取pushMode
        Integer pushModel = redisUtil.getByKeyAndField(RedisEnum.pushModel.getName(),
                RedisEnum.hospitalId_.getName() + pushVO.getHospitalId());
        switch (pushModel) {
            case 0:  // 使用新版推送
                return pushNewFac(pushVO, "0");
            case 1: // 使用5.0推送
                return processAggreate(pushVO);
            case 2:  // 走科室对应的模型，如果找不到，使用5.0推理
                List<Item> deptList = pushVO.getDept();
                String modelType = "0";
                if (ListUtil.isNotEmpty(deptList)) {
                    if (StringUtil.isNotBlank(deptList.get(0).getUniqueName())) {
                        modelType = redisUtil.getByKeyAndField(RedisEnum.deptPush.getName(),
                                deptList.get(0).getUniqueName());
                    }
                }
                if (modelType == null || "0".equals(modelType)) {
                    return processAggreate(pushVO);
                } else {
                    return pushNewFac(pushVO, modelType);
                }
            default: // 默认使用新版推送
                return pushNewFac(pushVO, "0");
        }
    }

    /**
     * 5.0 推送总入口
     *
     * @param pushVo
     * @return
     */
    public PushDTO processAggreate(PushVO pushVo) {
        // 年龄容错处理
        if (pushVo.getAgeNum() == null) {
            pushVo.setAgeNum(CoreUtil.convertAge(pushVo.getAge()));
        }
        PushDTO pushDTO = new PushDTO();
        pushDTO = this.pushFacIcss(pushVo);
        /*Integer pushMode = tranHospitalInfoFacade.getPushMode(pushVo.getHospitalId());
        //全科推送，走5.0
        if (pushMode.equals(DiseasePushTypeEnum.gp.getKey())) {
            pushDTO = this.pushFacIcss(pushVo);
        } else if (pushMode.equals(DiseasePushTypeEnum.specialty.getKey())) {
            //专科推送，走6.0
            //pushDTO = this.pushFac(pushVo);
            // 2021/2/22 同老楼确认专科也走5.0
            pushDTO = this.pushFacIcss(pushVo);
        }*/

        return pushDTO;
    }

    /**
     * 推送调icss5.0接口
     *
     * @param pushVo
     * @return
     */
    public PushDTO pushFacIcss(PushVO pushVo) {
        Map<String, Object> debug = new LinkedHashMap<>();
        long pushStart = System.currentTimeMillis();
        PushDTO pushDTO = pushProcess.processIcss(pushVo);
        CoreUtil.getDebugStr(pushStart, "推送耗时", debug);
        pushDTO.setDebug(debug);
        return pushDTO;
    }

    /**
     * 推送业务6.0
     *
     * @param
     * @return
     */
    public PushDTO pushNewFac(PushVO pushVo, String modelType) {
        PushDTO pushDTO = new PushDTO();
        Map<String, Object> debug = new LinkedHashMap<>();
        debug.put("大数据推送地址", pushUrl);

        // CRF模型处理
        long start = System.currentTimeMillis();
        WordCrfDTO wordCrfDTO = commonFacade.crf_process(pushVo, false);
        CoreUtil.getDebugStr(start, "模型处理耗时", debug);

        // 同义词转换
        long standStart = System.currentTimeMillis();
        commonFacade.wordStandConvert(wordCrfDTO);
        CoreUtil.getDebugStr(standStart, "标准词转换耗时", debug);
        ParamUtil.dealLis(wordCrfDTO.getLis());

        // 是否有界面诊断
        boolean hasDiseaseFlag = pushProcess.hasDisease(wordCrfDTO);
        if (hasDiseaseFlag) { // 有界面诊断，走诊断依据推送警惕，不推确诊和拟诊
            pushDTO = pushProcess.pushDiagnose(wordCrfDTO);
        } else { // 无界面诊断，诊断依据推送确诊、拟诊、警惕 + 大数据推送可能诊断
            try {
                Map<String, Object> invokeParams = new HashMap<>();
                invokeParams.put("wordCrfDTO", wordCrfDTO);
                invokeParams.put("pushNewVO", generatePushVo(wordCrfDTO, modelType));
                pushDTO = DataFacade.get("pushAll", invokeParams, PushDTO.class);
            } catch (Exception e) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "推送服务出错" + e.getMessage());
            }
        }

        // 诊断去重、合并处理、过滤
        dealDisease(pushDTO, wordCrfDTO, pushVo.getLength(), hasDiseaseFlag);

        // 通过诊断反推
        long reverseStart = System.currentTimeMillis();
        pushProcess.process(pushDTO, pushVo, wordCrfDTO);
        CoreUtil.getDebugStr(reverseStart, "反推业务耗时", debug);

        CoreUtil.getDebugStr(start, "总计耗时", debug);
        CoreUtil.getDebugObject("数据", debug, wordCrfDTO);
        pushDTO.getDebug().putAll(debug);

        // 添加既往诊断，不参与业务
        List<Diag> diagList = wordCrfDTO.getPastLabel().getDiags();
        List<String> pastDiagList = Lists.newArrayList();
        if (ListUtil.isNotEmpty(diagList)) {
            List<PushBaseDTO> pushBaseDTOList = Lists.newArrayList();
            for (Diag diag : diagList) {
                String diagName = diag.getName();
                if (diag.getNegative() == null && !pastDiagList.contains(diagName)) {
                    PushBaseDTO pushBaseDTO = new PushBaseDTO();
                    pushBaseDTO.setName(diagName);
                    pushBaseDTOList.add(pushBaseDTO);
                    pastDiagList.add(diagName);
                }
            }
            pushDTO.getDis().put(DiagnoseTypeEnum.pastDisease.getName(), pushBaseDTOList);
        }

        return pushDTO;
    }

    /**
     * 诊断去重、合并处理
     *
     * @param pushDTO
     */
    public void dealDisease(PushDTO pushDTO, WordCrfDTO wordCrfDTO, int length, boolean hasDiseaseFlag) {
        Map<String, List<PushBaseDTO>> dis = pushDTO.getDis();
        List<PushBaseDTO> possibleUnion = Lists.newArrayList(); // 可能诊断 = 确诊 + 拟诊 + 大数据推送诊断
        // 无界面诊断
        if (!hasDiseaseFlag) {
            List<PushBaseDTO> definite = dis.get(DiagnoseTypeEnum.definite.getName()); // 确诊
            List<PushBaseDTO> protocol = dis.get(DiagnoseTypeEnum.protocol.getName()); // 拟诊
            List<PushBaseDTO> bigDataPush = dis.get(DiagnoseTypeEnum.bigDataPush.getName()); // 大数据推送诊断
            // 合并可能诊断
            if (ListUtil.isNotEmpty(definite)) {
                possibleUnion.addAll(definite);
            }
            if (ListUtil.isNotEmpty(protocol)) {
                possibleUnion.addAll(protocol);
            }
            if (ListUtil.isNotEmpty(bigDataPush)) {
                possibleUnion.addAll(bigDataPush);
            }
            // 去重
            CoreUtil.removeRepeat(possibleUnion, "name");
        }
        // // 获取既往史否定诊断
        // List<String> negativeDis = CoreUtil.getPropertyWithFilter(wordCrfDTO.getPastLabel().getDiags(),
        //         "standName", "negative");
        // if (ListUtil.isNotEmpty(negativeDis)) { // 添加既往史否定诊断
        //     pushDTO.getDis().put("既往史否定诊断", pushProcess.generatePushBaseDTO(negativeDis, -1));
        //     // 可能诊断——剔除既往否定诊断
        //     filterDisease(dis.get(DiagnoseTypeEnum.possibleUnion.getName()), negativeDis);
        //     // 鉴别诊断——剔除既往否定诊断
        //     filterDisease(dis.get(DiseaseTypeEnum.identify.getName()), negativeDis);
        //     // 警惕——剔除既往否定诊断
        //     filterDisease(dis.get(DiagnoseTypeEnum.vigilant.getName()), negativeDis);
        // }
        // 可能诊断根据长度截取
        if (ListUtil.isNotEmpty(possibleUnion)) {
            possibleUnion = possibleUnion.size() > length ? possibleUnion.subList(0, length) : possibleUnion;
        }
        // 添加可能诊断
        dis.put(DiagnoseTypeEnum.possibleUnion.getName(), possibleUnion);
    }

    /**
     * 剔除既往否定诊断
     *
     * @param pushBaseDTOList
     * @param negativeDis
     */
    public void filterDisease(List<PushBaseDTO> pushBaseDTOList, List<String> negativeDis) {
        if (ListUtil.isEmpty(pushBaseDTOList) || ListUtil.isEmpty(negativeDis)) {
            return;
        }
        Iterator<PushBaseDTO> it = pushBaseDTOList.iterator();
        while (it.hasNext()) {
            PushBaseDTO bean = it.next();
            // 剔除既往否定诊断
            if (negativeDis.contains(bean.getName())) {
                it.remove();
            }
        }
    }

    /**
     * 生成新版推送入参
     *
     * @param wordCrfDTO
     * @param modelType  模型类型
     * @return
     */
    public PushNewVO generatePushVo(WordCrfDTO wordCrfDTO, String modelType) {
        PushNewVO pushNewVO = new PushNewVO();
        pushNewVO.setChief(wordCrfDTO.getChief());
        pushNewVO.setPresent(wordCrfDTO.getSymptom());
        pushNewVO.setAge(wordCrfDTO.getAgeNum());
        pushNewVO.setSex(wordCrfDTO.getSex());
        if (StringUtil.isNotBlank(modelType)) {
            pushNewVO.setHospitalType(modelType);
        }
        return pushNewVO;
    }

    /**
     * 手术随访计划（持续检验检查）推送
     *
     * @param pushPlanVO
     * @return
     */
    public PushPlanDTO pushPlan(PushPlanVO pushPlanVO) {
        Map<String, Object> debug = new LinkedHashMap<>();
        long l1 = System.currentTimeMillis();
        PushPlanDTO pushPlanDTO = new PushPlanDTO();
        if (pushPlanVO.getOperationName() != null) {
            WordCrfDTO wordCrfDTO = new WordCrfDTO();
            wordCrfDTO.setOperationName(pushPlanVO.getOperationName());
            // 标准词转换
            long standStart = System.currentTimeMillis();
            commonFacade.wordStandConvert(wordCrfDTO);
            CoreUtil.getDebugStr(standStart, "标准词转换耗时", debug);
            // 推送随访计划
            long pushStart = System.currentTimeMillis();
            pushPlanDTO = followupPlanInfoFacade.getFollowUpPlans(pushPlanVO.getOperationName(), pushPlanVO.getHospitalId());
            CoreUtil.getDebugStr(pushStart, "推送耗时", debug);
            pushPlanDTO.setDebug(debug);
        }
        CoreUtil.getDebugStr(l1, "本次调用总计耗时", debug);

        return pushPlanDTO;
    }

    /**
     * 测试新版推送
     *
     * @param pushNewVO
     * @return
     */
    public PushNewDTO testPushFac(PushNewVO pushNewVO) {
        return pushNewServiceClient.pushNew(pushNewVO);
    }
}
