package com.diagbot.process;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleConditionDTO;
import com.diagbot.dto.RuleExtDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.RuleTypeEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.facade.CommonFacade;
import com.diagbot.model.label.PacsLabel;
import com.diagbot.rule.LisRule;
import com.diagbot.rule.PacsRule;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.RuleVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * @Description: 危急值总入口
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class CriticalProcess {

    @Autowired
    LisRule lisRule;
    @Autowired
    PacsRule pacsRule;
    @Autowired
    CommonFacade commonFacade;

    /**
     * 处理业务——化验
     *
     * @param wordCrfDTO
     * @param res
     */
    public void processLis(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        List<BillMsg> criticalList = res.getCriticalValList();
        List<Lis> lisList = wordCrfDTO.getLis();
        for (Lis lis : lisList) {
            RuleVO ruleVO = getRuleVO(lis);
            // 1、获取危急值相关数据
            if (ruleVO != null) {
                RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
                if (ruleExtDTO == null) {
                    continue;
                }
                List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
                for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                    List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                    // 2、规则匹配
                    lisRule.critical(lis, ruleBaseDTOList, criticalList, wordCrfDTO);
                }
            }
        }
    }

    /**
     * 处理业务——辅检
     *
     * @param wordCrfDTO
     * @param res
     */
    public void processPacs(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        List<BillMsg> criticalList = res.getCriticalValList();
        PacsLabel pacsLabel = wordCrfDTO.getPacsLabel();
        List<Item> itemList = pacsLabel.getRes();
        Set<String> set = new LinkedHashSet<>();
        List<String> msg = new LinkedList<>();
        for (Item item : itemList) {
            RuleVO ruleVO = getRuleVO(item);
            if (ruleVO != null) {
                // 1、获取危急值相关数据
                RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
                // 如果有完全包含关系，留下最长的一条，例如：右中间支气管异物， 支气管异物，异物，
                // 最终只提示：右中间支气管异物
                if (ruleExtDTO != null) {
                    set.add(ruleExtDTO.getLibName());
                }
            }
        }
        for (String s : set) {
            boolean flag = true;
            for (String s1 : set) {
                if (s1.contains(s) && !s1.equals(s)) {
                    flag = false;
                    break;
                }
            }
            if (flag) {
                msg.add(s);
            }
        }
        pacsRule.critical(msg, criticalList);
    }

    /**
     * 获取危急值入参——化验
     *
     * @param lis
     * @return
     */
    public RuleVO getRuleVO(Lis lis) {
        if (StringUtil.isNotBlank(lis.getUniqueName())) {
            RuleVO ruleVO = new RuleVO();
            ruleVO.setInputName(lis.getName());
            ruleVO.setLibName(lis.getUniqueName());
            ruleVO.setLibType(LexiconEnum.LisSubName.getKey());
            ruleVO.setLibTypeName(TypeEnum.lis.getName());
            ruleVO.setDateValue(lis.getDateValue());
            ruleVO.setSuffix(String.valueOf(RuleTypeEnum.critical.getKey()));
            return ruleVO;
        }
        return null;
    }

    /**
     * 获取危急值入参——辅检
     *
     * @param item
     * @return
     */
    public RuleVO getRuleVO(Item item) {
        if (StringUtil.isNotBlank(item.getUniqueName())) {
            RuleVO ruleVO = new RuleVO();
            ruleVO.setInputName(item.getName());
            ruleVO.setLibName(item.getUniqueName());
            ruleVO.setLibType(LexiconEnum.PacsResult.getKey());
            ruleVO.setLibTypeName(TypeEnum.pacs.getName());
            ruleVO.setDateValue(item.getDateValue());
            ruleVO.setSuffix(String.valueOf(RuleTypeEnum.critical.getKey()));
            return ruleVO;
        }
        return null;
    }
}
