package com.diagbot.process;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleConditionDTO;
import com.diagbot.dto.RuleExtDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.RuleTypeEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.facade.CommonFacade;
import com.diagbot.model.label.PacsLabel;
import com.diagbot.rule.CommonRule;
import com.diagbot.rule.GroupRule;
import com.diagbot.rule.LisRule;
import com.diagbot.rule.PacsRule;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.RuleVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description: 其他提示总入口
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class OtherTipProcess {

    @Autowired
    GroupRule groupRule;
    @Autowired
    CommonRule commonRule;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    LisRule lisRule;
    @Autowired
    PacsRule pacsRule;

    /**
     * 处理业务——化验
     *
     * @param wordCrfDTO
     * @param res
     */
    public void processLis(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        List<Lis> lisList = wordCrfDTO.getLis();
        List<BillMsg> otherList = res.getOtherList();
        for (Lis lis : lisList) {
            RuleVO ruleVO = getRuleVO(lis, RuleTypeEnum.other.getKey());
            // 1、获取相关数据
            if (ruleVO != null) {
                RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
                if (ruleExtDTO == null) {
                    continue;
                }
                List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
                for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                    List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                    // 2、规则匹配
                    lisRule.otherLis(lis, ruleBaseDTOList, otherList, wordCrfDTO);
                }
            }
        }
    }

    /**
     * 处理业务——辅检
     *
     * @param wordCrfDTO
     * @param res
     */
    public void processPacs(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        PacsLabel pacsLabel = wordCrfDTO.getPacsLabel();
        List<Item> itemList = pacsLabel.getRes();
        List<BillMsg> otherList = res.getOtherList();
        for (Item item : itemList) {
            RuleVO ruleVO = getRuleVO(item);
            // 1、获取相关数据
            if (ruleVO != null) {
                RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
                if (ruleExtDTO == null) {
                    continue;
                }
                List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
                for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                    List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                    // 2、规则匹配
                    pacsRule.otherPacs(ruleBaseDTOList, ruleConditionDTO.getMsg(), otherList, wordCrfDTO);
                }
            }
        }
    }

    /**
     * 处理业务——输血
     *
     * @param wordCrfDTO
     * @param res
     */
    public void processTransfusion(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        List<Lis> lisList = wordCrfDTO.getLis();
        List<BillMsg> otherList = res.getOtherList();
        for (Lis lis : lisList) {
            RuleVO ruleVO = getRuleVO(lis, RuleTypeEnum.transfusion.getKey());
            // 1、获取相关数据
            if (ruleVO != null) {
                RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
                if (ruleExtDTO == null) {
                    continue;
                }
                List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
                for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                    List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                    // 2、规则匹配
                    lisRule.otherTransfusion(lis, ruleBaseDTOList, ruleConditionDTO.getMsg(), otherList, wordCrfDTO);
                }
            }
        }
    }

    /**
     * 获取其他值提醒入参——辅检
     *
     * @param bean
     * @return
     */
    public RuleVO getRuleVO(Item bean) {
        if (StringUtil.isNotBlank(bean.getUniqueName())) {
            RuleVO ruleVO = new RuleVO();
            ruleVO.setInputName(bean.getName());
            ruleVO.setLibName(bean.getUniqueName());
            ruleVO.setLibType(LexiconEnum.PacsResult.getKey());
            ruleVO.setLibTypeName(TypeEnum.pacs.getName());
            ruleVO.setDateValue(bean.getDateValue());
            ruleVO.setSuffix(String.valueOf(RuleTypeEnum.other.getKey()));
            return ruleVO;
        }
        return null;
    }

    /**
     * 获取其他值提醒入参——化验
     *
     * @param lis
     * @return
     */
    public RuleVO getRuleVO(Lis lis, Integer key) {
        if (StringUtil.isNotBlank(lis.getUniqueName())) {
            RuleVO ruleVO = new RuleVO();
            ruleVO.setInputName(lis.getName());
            ruleVO.setLibName(lis.getUniqueName());
            ruleVO.setLibType(LexiconEnum.LisSubName.getKey());
            ruleVO.setLibTypeName(TypeEnum.lis.getName());
            ruleVO.setDateValue(lis.getDateValue());
            ruleVO.setSuffix(String.valueOf(key));
            return ruleVO;
        }
        return null;
    }
}
