package com.diagbot.process;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.client.IcssPushServiceClient;
import com.diagbot.dto.BaseDiagnoseDTO;
import com.diagbot.dto.DiagnoseDTO;
import com.diagbot.dto.NeoPushDTO;
import com.diagbot.dto.PushBaseDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.ReverseDTO;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.TreatDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.dto.icss.FeatureRate;
import com.diagbot.dto.icss.Response;
import com.diagbot.dto.icss.ResponseData;
import com.diagbot.entity.PreResult;
import com.diagbot.enums.BaseDiagnoseTypeEnum;
import com.diagbot.enums.DiagnoseTypeEnum;
import com.diagbot.enums.DiseaseTypeEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.MedicalAdviceEnum;
import com.diagbot.enums.PushRelationTypeEnum;
import com.diagbot.enums.RedisEnum;
import com.diagbot.enums.RelationshipEnum;
import com.diagbot.enums.StandConvertEnum;
import com.diagbot.facade.CommonFacade;
import com.diagbot.facade.ConceptInfoFacade;
import com.diagbot.facade.KlDiagnoseFacade;
import com.diagbot.facade.KlRelationFacade;
import com.diagbot.facade.LisConfigFacade;
import com.diagbot.facade.NeoFacade;
import com.diagbot.facade.TranLisConfigIcssFacade;
import com.diagbot.model.entity.Clinical;
import com.diagbot.rule.AgeRule;
import com.diagbot.rule.CommonRule;
import com.diagbot.rule.GroupRule;
import com.diagbot.rule.LisRule;
import com.diagbot.rule.VitalRule;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RedisUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.DiagnoseVO;
import com.diagbot.vo.DiseaseItemVO;
import com.diagbot.vo.IcssVo;
import com.diagbot.vo.NeoPushVO;
import com.diagbot.vo.PushVO;
import com.diagbot.vo.ReverseVO;
import com.diagbot.vo.neoPushEntity.Diag;
import com.diagbot.vo.neoPushEntity.DiagVo;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author kwz
 * @date 2020/8/17
 * @time 19:42
 */
@Component
@Slf4j
public class PushProcess {

    @Autowired
    NeoFacade neoFacade;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    TranLisConfigIcssFacade tranLisConfigIcssFacade;
    @Autowired
    IcssPushServiceClient icssPushServiceClient;
    @Autowired
    RedisUtil redisUtil;
    @Autowired
    KlRelationFacade klRelationFacade;
    @Autowired
    CommonRule commonRule;
    @Autowired
    LisRule lisRule;
    @Autowired
    KlDiagnoseFacade klDiagnoseFacade;
    @Autowired
    AgeRule ageRule;
    @Autowired
    VitalRule vitalRule;
    @Autowired
    GroupRule groupRule;
    @Autowired
    ConceptInfoFacade conceptInfoFacade;
    @Autowired
    LisConfigFacade lisConfigFacade;

    public PushDTO processIcss(PushVO pushVo) {

        int length = pushVo.getLength();
        Map<String, List<String>> typeWords = Maps.newHashMap();
        List<String> ruleTypeList = Arrays.asList(pushVo.getFeatureType().split(","));
        List<String> pushDiags = Lists.newArrayList();
        Map<String, Integer> diag_sign = new LinkedHashMap<>();
        PushDTO pushDTO = new PushDTO();
        IcssVo icssVo = pushVo2IcssVo(pushVo);
        //        String s = JSONObject.toJSONString(icssVo);
        if ((StringUtils.isNotBlank(icssVo.getChief()) && icssVo.getChief().trim().equals("呼吸困难"))
                || (StringUtils.isNotBlank(icssVo.getSymptom()) && icssVo.getSymptom().trim().equals("呼吸困难"))) {
            icssVo.setChief("呼吸困难。");
        }
        Response<ResponseData> annotation = icssPushServiceClient.getAnnotation(icssVo);
        if (annotation != null) {
            ResponseData data = annotation.getData();
            if (data != null) {
                Map<String, Map<String, String>> inputs = data.getInputs();
                if (MapUtils.isNotEmpty(inputs)) {
                    List<String> words = new ArrayList<>(inputs.keySet());
                    typeWords.put(StandConvertEnum.symptom.toString(), words);
                    typeWords.put(StandConvertEnum.disease.toString(), words);
                    typeWords.put(StandConvertEnum.pacs.toString(), words);
                    typeWords.put(StandConvertEnum.lis.toString(), words);
                    typeWords.put(StandConvertEnum.vital.toString(), words);
                }
                List<FeatureRate> dis = data.getDis();
                if (ListUtil.isNotEmpty(dis)) {
                    pushDiags = dis.stream().filter(x -> x.getDesc() == null || x.getDesc().contains("确诊") || x.getDesc().contains("拟诊"))
                            .map(x -> {
                                        String diag = x.getFeatureName();
                                        Integer sign = 0;
                                        Map<String, Object> diag_signs = (Map<String, Object>) redisUtil.get(RedisEnum.diseaseCorrespond.getName() + x.getFeatureName());
                                        if (MapUtils.isNotEmpty(diag_signs)) {
                                            diag = (String) diag_signs.get("diag_icd");
                                            sign = (Integer) diag_signs.get("sign");
                                        }
                                        diag_sign.put(diag, sign);
                                        return diag;
                                    }
                            )
                            .collect(Collectors.toList());
                }
            }
        }
        Item diaeaseName = pushVo.getDiseaseName();
        Map<String, List<PushBaseDTO>> dis = new HashMap<>();
        if (ruleTypeList.contains("10") && diaeaseName != null) {
            List<TreatDTO> collect = new ArrayList<>();
            TreatDTO treatDTO = new TreatDTO();
            treatDTO.setName(diaeaseName.getName());
            String uniqueName = diaeaseName.getUniqueName();
            uniqueName = StringUtils.isNotBlank(uniqueName) ? uniqueName : diaeaseName.getName();
            treatDTO.setUniqueName(uniqueName);
            collect.add(treatDTO);
            pushDTO.setTreat(collect);
        }
        Map<String, List<String>> reverseInfoByDisName = null;
        //推送治疗
        if (diaeaseName != null) {
            String uniqueName = diaeaseName.getUniqueName();
            uniqueName = StringUtils.isNotBlank(uniqueName) ? uniqueName : diaeaseName.getName();
            //reverseInfoByDisName = neoFacade.reverseInfoByDisName(uniqueName);
            //查询疾病相关的项目
            DiseaseItemVO diseaseItemVO = new DiseaseItemVO();
            diseaseItemVO.setDiseaseName(uniqueName);
            Map<Long, List<String>> items = klRelationFacade.getItemByDiseaseMap(diseaseItemVO);
            if (MapUtils.isNotEmpty(items)) {
                if (ruleTypeList.contains("8") && ListUtil.isNotEmpty(items.get(PushRelationTypeEnum.DRUG.getKey()))) {
                    pushDTO.setMedicines(items.get(PushRelationTypeEnum.DRUG.getKey()).stream().map(x -> {
                        PushBaseDTO pushBaseDTO = new PushBaseDTO();
                        pushBaseDTO.setName(x);
                        return pushBaseDTO;
                    }).collect(Collectors.toList()));
                }
                if (ruleTypeList.contains("9") && ListUtil.isNotEmpty(items.get(PushRelationTypeEnum.OPERATION.getKey()))) {
                    pushDTO.setOperations(items.get(PushRelationTypeEnum.OPERATION.getKey()).stream().map(x -> {
                        PushBaseDTO pushBaseDTO = new PushBaseDTO();
                        pushBaseDTO.setName(x);
                        return pushBaseDTO;
                    }).collect(Collectors.toList()));
                }
                if (ruleTypeList.contains("13") && ListUtil.isNotEmpty(items.get(PushRelationTypeEnum.NURSE.getKey()))) {
                    pushDTO.setNurse(items.get(PushRelationTypeEnum.NURSE.getKey()).stream().map(x -> {
                        PushBaseDTO pushBaseDTO = new PushBaseDTO();
                        pushBaseDTO.setName(x);
                        return pushBaseDTO;
                    }).collect(Collectors.toList()));
                }
            }
        }
        //        if (reverseInfoByDisName != null) {
        //            if (ruleTypeList.contains("8") && reverseInfoByDisName.containsKey(StaticTypeEnum.DRUG.getName())) {
        //                pushDTO.setMedicines(reverseInfoByDisName.get(StaticTypeEnum.DRUG.getName()).stream().map(x -> {
        //                    PushBaseDTO pushBaseDTO = new PushBaseDTO();
        //                    pushBaseDTO.setName(x);
        //                    return pushBaseDTO;
        //                }).collect(Collectors.toList()));
        //            }
        //            if (ruleTypeList.contains("9") && reverseInfoByDisName.containsKey(StaticTypeEnum.OPT.getName())) {
        //                pushDTO.setOperations(reverseInfoByDisName.get(StaticTypeEnum.OPT.getName()).stream().map(x -> {
        //                    PushBaseDTO pushBaseDTO = new PushBaseDTO();
        //                    pushBaseDTO.setName(x);
        //                    return pushBaseDTO;
        //                }).collect(Collectors.toList()));
        //            }
        //        }
        if (ListUtil.isNotEmpty(icssVo.getDiagOrder())) {
            List<PreResult> diagOrder = icssVo.getDiagOrder();
            //把第一个推送出来的诊断set到diagOrder中，再反推
            NeoPushVO pushVO = new NeoPushVO();
            DiagVo diagVo = new DiagVo();
            diagVo.setDiags(diagOrder.stream().map(x -> {
                String name = x.getName();
                Diag diag = new Diag();
                Map<String, Object> diag_signs = (Map<String, Object>) redisUtil.get(RedisEnum.diseaseCorrespond.getName() + name);
                name = MapUtils.isNotEmpty(diag_signs) ? (String) diag_signs.get("diag_icd") : name;
                diag.setName(name);
                return diag;
            }).collect(Collectors.toList()));
            pushVO.setDiagVo(diagVo);
            pushVO.setAgeNum(pushVo.getAgeNum());
            pushVO.setSex(pushVo.getSex());
            reversePushPackage(length, pushDTO, ruleTypeList, typeWords, pushVO, dis);
        }
        try {
            boolean allFieldNull = isAllFieldNull(pushDTO);
            if (allFieldNull) {
                if (ListUtil.isNotEmpty(pushDiags)) {
                    //先把所有疾病推出来
                    if (ruleTypeList.contains("7")) {
                        if (MapUtils.isNotEmpty(diag_sign)) {
                            Map.Entry<String, Integer> first = diag_sign.entrySet().stream().findFirst().get();
                            if (first.getValue() == 1) {
                                List<String> firstdiag = Lists.newArrayList(first.getKey());
                                List<PushBaseDTO> collect_highrisk = firstdiag.stream().map(x -> {
                                    PushBaseDTO pushBaseDTO = new PushBaseDTO();
                                    pushBaseDTO.setName(x);
                                    return pushBaseDTO;
                                }).collect(Collectors.toList());
                                dis.put(DiseaseTypeEnum.highriskDis.getName(), collect_highrisk);
                            }

                        }

                        List<PushBaseDTO> diseeases = pushDiags.stream().map(x -> {
                            PushBaseDTO pushBaseDTO = new PushBaseDTO();
                            pushBaseDTO.setName(x);
                            return pushBaseDTO;
                        }).collect(Collectors.toList());
                        diseeases = diseeases.subList(0, diseeases.size() >= length ? length : diseeases.size());
                        dis.put(DiseaseTypeEnum.possibleDis.getName(), diseeases);
                        pushDTO.setDis(dis);

                    }
                    //把第一个推送出来的诊断set到diagOrder中，再反推
                    NeoPushVO pushVO = new NeoPushVO();
                    DiagVo diagVo = new DiagVo();
                    diagVo.setDiags(pushDiags.stream().map(x -> {
                        Diag diag = new Diag();
                        diag.setName(x);
                        return diag;
                    }).collect(Collectors.toList()));
                    pushVO.setDiagVo(diagVo);
                    pushVO.setAgeNum(pushVo.getAgeNum());
                    pushVO.setSex(pushVo.getSex());
                    //调用反推
                    reversePushPackage(length, pushDTO, ruleTypeList, typeWords, pushVO, dis);

                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }


        return pushDTO;
    }

    //判断该对象是否: 返回true表示所有属性为null  返回false表示不是所有属性都是null
    public static boolean isAllFieldNull(Object obj) throws Exception {
        List<String> params = Lists.newArrayList("operations", "medicines", "treat");
        Class stuCla = (Class) obj.getClass();// 得到类对象
        Field[] fs = stuCla.getDeclaredFields();//得到属性集合
        boolean flag = true;
        for (Field f : fs) {//遍历属性
            f.setAccessible(true); // 设置属性是可以访问的(私有的也可以)
            String name = f.getName();
            Object val = f.get(obj);// 得到此属性的值
            if (val != null && !params.contains(name)) {//只要有1个属性不为空,那么就不是所有的属性值都为空
                flag = false;
                break;
            }
        }
        return flag;
    }

    //判断该对象是否为空，返回true表示所有属性为null  返回false表示不是所有属性都是null
    public static boolean isAllFieldNull(PushDTO pushDTO) {
        if (ListUtil.isNotEmpty(pushDTO.getSymptom())
                || ListUtil.isNotEmpty(pushDTO.getVital())
                || ListUtil.isNotEmpty(pushDTO.getLis())
                || ListUtil.isNotEmpty(pushDTO.getPacs())
                || (pushDTO.getDis() != null && pushDTO.getDis().size() > 0)
                || ListUtil.isNotEmpty(pushDTO.getComplications())) {
            return false;
        }
        return true;
    }

    /**
     * pushVo转Icss入参
     *
     * @param pushVo
     * @return
     */
    public IcssVo pushVo2IcssVo(PushVO pushVo) {
        IcssVo icssVo = new IcssVo();
        icssVo.setAge(new Double(pushVo.getAgeNum()).intValue());
        String sexs = "";
        Integer sex = pushVo.getSex();
        if (sex == 1) {
            sexs = "M";
        } else if (sex == 2) {
            sexs = "F";
        }
        icssVo.setSex(sexs);
        icssVo.setChief(pushVo.getChief());
        icssVo.setSymptom(pushVo.getSymptom());
        icssVo.setVital(pushVo.getVital());
        icssVo.setPasts(pushVo.getPasts());
        icssVo.setPersonal(pushVo.getPersonal());
        icssVo.setFeatureType(pushVo.getFeatureType() + ",7");

        List<Lis> lis = pushVo.getLis();
        lis = tranLisConfigIcssFacade.addLisUniqueName(lis, "A001");
        List<PreResult> lis_icss = lis.stream().map(x -> {
            PreResult instantiate = BeanUtils.instantiate(PreResult.class);
            BeanUtils.copyProperties(x, instantiate);
            instantiate.setValue(x.getValue() == null ? "" : x.getValue().toString());
            instantiate.setMaxValue(x.getMaxValue() == null ? "" : x.getMaxValue().toString());
            instantiate.setMinValue(x.getMinValue() == null ? "" : x.getMinValue().toString());
            return instantiate;
        }).collect(Collectors.toList());
        icssVo.setLis(lis_icss);

        List<Pacs> pacs = pushVo.getPacs();
        if (ListUtil.isNotEmpty(pacs)) {
            List<PreResult> pacs_icss = pacs.stream().map(x -> {
                PreResult instantiate = new PreResult();
                BeanUtils.copyProperties(x, instantiate);
                return instantiate;
            }).collect(Collectors.toList());
            icssVo.setPacs(pacs_icss);
        }

        if (ListUtil.isNotEmpty(pushVo.getDiag())) {
            List<PreResult> diagOrder = pushVo.getDiag().stream().map(x -> {
                String name = x.getName();
                PreResult preResult = new PreResult();
                preResult.setName(name);
                preResult.setValue(name);
                return preResult;
            }).collect(Collectors.toList());
            icssVo.setDiagOrder(diagOrder);
        }
        return icssVo;
    }

    /**
     * 根据诊断反推(根据性别和年龄过滤，并根据界面已有信息过滤，根据长度截取)
     *  推送逻辑：第一诊断查询其他推送项（除去治疗相关），选中诊断查询治疗相关的推送项
     *  治疗相关：药品、手术、护理、一般治疗
     *
     * @param pushDTO
     * @param pushVo
     * @param wordCrfDTO
     * @return
     */
    public void process(PushDTO pushDTO, PushVO pushVo, WordCrfDTO wordCrfDTO) {
        // 一般治疗由界面选中诊断diseaseName获取。core服务处理，这里只返回结构体
        Item selectDis = wordCrfDTO.getDiseaseName(); // 选中诊断
        Boolean hasDiseaseNameFlag = hasDiseaseName(selectDis);
        String selectDisName = hasDiseaseNameFlag ? selectDis.getUniqueName() : "";
        if (hasDiseaseNameFlag) {
            TreatDTO treatDTO = new TreatDTO();
            BeanUtil.copyProperties(selectDis, treatDTO);
            pushDTO.getTreat().add(treatDTO);
        }
        // 标识选中诊断和第一诊断名称是否相同。这里做了优化处理，如果第一诊断和选中诊断相同，反推只需查询一次
        Boolean diseaseNameEqFirstFirstFlag = false;
        if (hasDiseaseNameFlag && ListUtil.isNotEmpty(wordCrfDTO.getDiag())
                && selectDisName.equals(wordCrfDTO.getDiag().get(0).getUniqueName())) {
            diseaseNameEqFirstFirstFlag = true;
        }

        // 第一诊断反推
        List<String> featureType = Lists.newArrayList(pushVo.getFeatureType().split(",")); // 需要获取的类型
        Map<String, List<PushBaseDTO>> dis = pushDTO.getDis();
        ReverseVO reverseVO = generateReverseVO(wordCrfDTO, featureType, dis, diseaseNameEqFirstFirstFlag);

        // 输出调试信息
        if (StringUtil.isBlank(selectDisName)) {
            pushDTO.getDebug().put("选中诊断推送【一般治疗、药品、手术、护理】", "无"); // 调试信息
        } else {
            pushDTO.getDebug().put("选中诊断推送【一般治疗、药品、手术、护理】", selectDisName); // 调试信息
        }
        if (StringUtil.isBlank(reverseVO.getLibName())) {
            pushDTO.getDebug().put("第一诊断推送其他信息", "无"); // 调试信息
            return;
        } else {
            pushDTO.getDebug().put("第一诊断推送其他信息", reverseVO.getLibName()); // 调试信息
        }

        List<ReverseDTO> reverseDTOList = conceptInfoFacade.getReverseFac(reverseVO);
        // 选中诊断反推
        if (hasDiseaseNameFlag && !diseaseNameEqFirstFirstFlag) {
            reverseVO.setLibName(selectDisName);
            // 关联类型
            List<Integer> relationType = new ArrayList<>();
            if (featureType.contains("8")) {
                relationType.add(RelationshipEnum.R506.getKey()); // 药品
            }
            if (featureType.contains("9")) {
                relationType.add(RelationshipEnum.R507.getKey()); // 手术
            }
            if (featureType.contains("13")) {
                relationType.add(RelationshipEnum.R510.getKey()); // 护理
            }
            reverseVO.setRelationType(relationType);
            List<ReverseDTO> reverseDTOList1 = conceptInfoFacade.getReverseFac(reverseVO);
            reverseDTOList.addAll(reverseDTOList1);
        }

        // 获取各个类型的界面过滤元素
        Map<String, List<String>> filterMap = getFilterMap(wordCrfDTO);
        // 设置推送信息（过滤已有）
        setPushItem(pushDTO, featureType, filterMap, reverseDTOList, pushVo.getLength());
    }

    /**
     * 是否有选中诊断
     *
     * @param item
     * @return
     */
    public Boolean hasDiseaseName(Item item) {
        if (item != null && StringUtil.isNotBlank(item.getUniqueName())) {
            return true;
        }
        return false;
    }

    /**
     * 根据诊断依据推送疾病总入口
     *
     * @param wordCrfDTO
     * @return
     */
    public PushDTO pushDiagnose(WordCrfDTO wordCrfDTO) {
        long start = System.currentTimeMillis();
        PushDTO pushDTO = null;
        try {
            pushDTO = pushDiagnoseProcess(wordCrfDTO);
        } catch (Exception e) {
            log.error("【诊断依据推理出错】", e);
            CoreUtil.getDebugStr("【诊断依据推理出错】", e, pushDTO.getDebug());
        }
        CoreUtil.getDebugStr(start, "推送诊断耗时_诊断依据", pushDTO.getDebug());
        return pushDTO;
    }

    /**
     * 根据诊断依据推送疾病
     *
     * @param wordCrfDTO
     */
    public PushDTO pushDiagnoseProcess(WordCrfDTO wordCrfDTO) {
        PushDTO pushDTO = new PushDTO();
        // 读取所有的诊断依据规则——已去重
        List<BaseDiagnoseDTO> diagnoselist = redisUtil.get(RedisEnum.allBaseDiagnose.getName());
        if (ListUtil.isEmpty(diagnoselist)) {
            return pushDTO;
        }
        // 匹配的诊断依据规则
        List<Long> baseIdList = new ArrayList<>();

        /**---------------------------数据统一处理开始--------------------------------- */
        // 【临床表现数据来源】（主诉、现病史）
        List<Clinical> clinicals = commonFacade.getClinicalSource(wordCrfDTO);
        /**---------------------------数据统一处理结束--------------------------------- */

        // 规则处理
        for (BaseDiagnoseDTO base : diagnoselist) {
            RuleBaseDTO ruleBaseDTO = new RuleBaseDTO();
            BeanUtil.copyProperties(base, ruleBaseDTO);
            String baseIds = base.getIds();
            switch (BaseDiagnoseTypeEnum.getEnum(base.getBaseType())) {
                case equals: // 等于术语本身
                    switch (LexiconEnum.getEnum(base.getBaseLibType())) {
                        case Symptom: // 症状
                            commonRule.compareNameWithPush(clinicals, ruleBaseDTO, baseIdList, baseIds);
                            break;
                        case Disease: // 疾病【病史】
                            commonRule.compareNameWithPush(wordCrfDTO.getPastLabel().getDiags(), ruleBaseDTO, baseIdList, baseIds);
                            break;
                        case VitalResult: // 体格检查结果
                            vitalRule.push(wordCrfDTO.getVitalLabel(), ruleBaseDTO, baseIdList, baseIds);
                            break;
                        case Group: // 人群
                            groupRule.push(wordCrfDTO, ruleBaseDTO, baseIdList, baseIds);
                            break;
                        case PacsResult: // 辅助检查结果
                            commonRule.compareItemWithPush(wordCrfDTO.getPacsLabel().getRes(), ruleBaseDTO, baseIdList, baseIds);
                            break;
                    }
                    break;
                case compare: // 比较
                    switch (LexiconEnum.getEnum(base.getBaseLibType())) {
                        case LisSubName: // 化验
                            lisRule.push(wordCrfDTO.getLis(), ruleBaseDTO, baseIdList, baseIds);
                            break;
                        case Age: // 年龄
                            ageRule.push(wordCrfDTO, ruleBaseDTO, baseIdList, baseIds);
                            break;
                        case Vital: // 体格检查项目
                            vitalRule.push(wordCrfDTO.getVitalLabel(), ruleBaseDTO, baseIdList, baseIds);
                            break;
                    }
                    break;
                case regexChiefSymptom: // 主诉现病史正则
                    commonRule.getRegexResPush(wordCrfDTO.getChief() + wordCrfDTO.getSymptom(),
                            base.getBaseEqValue(), baseIdList, baseIds);
                    break;
                case regexPast: // 既往史正则
                    commonRule.getRegexResPush(wordCrfDTO.getPasts() + wordCrfDTO.getPersonal() + wordCrfDTO.getMenstrual() + wordCrfDTO.getMarital() + wordCrfDTO.getFamily(),
                            base.getBaseEqValue(), baseIdList, baseIds);
                    break;
            }
        }
        // 根据诊断依据推送诊断
        Collections.sort(baseIdList); // id排序，方便调试查看
        pushDTO.getDebug().put("诊断依据ID列表", StringUtils.join(baseIdList, ","));
        Map<String, List<PushBaseDTO>> pushDiagnoseDTO = setPushDiagnose(wordCrfDTO, baseIdList);
        pushDTO.setDis(pushDiagnoseDTO);
        return pushDTO;
    }

    /**
     * 根据诊断依据推送诊断
     *
     * @param wordCrfDTO
     * @param baseIdList
     */
    public Map<String, List<PushBaseDTO>> setPushDiagnose(WordCrfDTO wordCrfDTO, List<Long> baseIdList) {
        Map<String, List<PushBaseDTO>> pushDiagnoseDTO = new LinkedHashMap<>();
        // 设置诊断
        if (ListUtil.isNotEmpty(baseIdList)) {
            DiagnoseVO diagnoseVO = new DiagnoseVO();
            diagnoseVO.setAge(wordCrfDTO.getAgeNum());
            diagnoseVO.setSexType(wordCrfDTO.getSex());
            diagnoseVO.setIds(baseIdList);
            List<DiagnoseDTO> diagnoseDTOList = klDiagnoseFacade.getDiagnoseByIds(diagnoseVO);
            List<PushBaseDTO> protocolList = new ArrayList<>(); // 拟诊
            List<PushBaseDTO> definiteList = new ArrayList<>(); // 确诊
            List<PushBaseDTO> vigilantList = new ArrayList<>(); // 警惕

            for (DiagnoseDTO diagnoseDTO : diagnoseDTOList) {
                PushBaseDTO pushBaseDTO = new PushBaseDTO();
                pushBaseDTO.setName(diagnoseDTO.getLibName());
                switch (DiagnoseTypeEnum.getEnum(diagnoseDTO.getConditionType())) {
                    case protocol: // 拟诊
                        protocolList.add(pushBaseDTO);
                        break;
                    case definite: // 确诊
                        definiteList.add(pushBaseDTO);
                        break;
                    case vigilant: // 警惕
                        vigilantList.add(pushBaseDTO);
                        break;
                }
            }
            // 无界面诊断，推送拟诊和确诊
            if (!hasDisease(wordCrfDTO)) {
                if (ListUtil.isNotEmpty(protocolList)) {
                    pushDiagnoseDTO.put("拟诊", protocolList);
                }
                if (ListUtil.isNotEmpty(definiteList)) {
                    pushDiagnoseDTO.put("确诊", definiteList);
                }
            }
            if (ListUtil.isNotEmpty(vigilantList)) {
                pushDiagnoseDTO.put("警惕", vigilantList);
            }
        }
        return pushDiagnoseDTO;
    }

    private void setPushVo(NeoPushVO pushVO, List<String> push) {
        String firstDisease = push.get(0);
        List<Diag> diags = new ArrayList<>();
        Diag diag = new Diag();
        diag.setName(firstDisease);
        diags.add(diag);
        DiagVo diagVo = new DiagVo();
        diagVo.setDiags(diags);
        pushVO.setDiagVo(diagVo);
    }

    /**
     * 获取各个类型的界面过滤元素
     *
     * @param wordCrfDTO
     * @return
     */
    public Map<String, List<String>> getFilterMap(WordCrfDTO wordCrfDTO) {
        Map<String, List<String>> filterMap = new HashMap<>();
        // 主诉症状
        dealFilter(filterMap, StandConvertEnum.symptom.toString(),
                CoreUtil.getByPropertyName(wordCrfDTO.getChiefLabel().getClinicals(), "name", "standName"));
        // 现病史症状
        dealFilter(filterMap, StandConvertEnum.symptom.toString(),
                CoreUtil.getByPropertyName(wordCrfDTO.getPresentLabel().getClinicals(), "name", "standName"));
        // 查体结果
        dealFilter(filterMap, StandConvertEnum.vital.toString(),
                CoreUtil.getByPropertyName(wordCrfDTO.getVitalLabel().getClinicals(), "name", "standName"));
        // 检查
        dealFilter(filterMap, StandConvertEnum.pacs.toString(),
                CoreUtil.getByPropertyName(wordCrfDTO.getPacs(), "name", "uniqueName"));
        // 药品
        dealFilter(filterMap, StandConvertEnum.drug.toString(),
                CoreUtil.getByPropertyName(wordCrfDTO.getDrug(), "name", "uniqueName"));
        // 手术
        dealFilter(filterMap, StandConvertEnum.operation.toString(),
                CoreUtil.getByPropertyName(wordCrfDTO.getOperation(), "name", "uniqueName"));
        // 诊断
        dealFilter(filterMap, StandConvertEnum.disease.toString(),
                CoreUtil.getByPropertyName(wordCrfDTO.getDiag(), "name", "uniqueName"));
        // 化验——获取过滤化验套餐名
        dealFilter(filterMap, StandConvertEnum.lis.toString(), lisConfigFacade.getMappingLisItem(wordCrfDTO));
        return filterMap;
    }

    /**
     * 根据类型生成过滤词
     *
     * @param map
     * @param key
     * @param filteString
     */
    public void dealFilter(Map<String, List<String>> map, String key, List<String> filteString) {
        if (ListUtil.isEmpty(filteString)) {
            return;
        }
        if (ListUtil.isNotEmpty(map.get(key))) { // 有值就追加
            map.get(key).addAll(filteString);
        } else { // 无值就初始化
            map.put(key, filteString);
        }
    }

    /**
     * 设置其他推送信息
     *
     * @param pushDTO
     * @param featureType
     * @param filterMap
     * @param reverseDTOList
     */
    public void setPushItem(PushDTO pushDTO, List<String> featureType, Map<String, List<String>> filterMap, List<ReverseDTO> reverseDTOList, int length) {
        Map<String, List<PushBaseDTO>> dis = pushDTO.getDis();
        Map<Integer, List<ReverseDTO>> map = EntityUtil.makeEntityListMap(reverseDTOList, "sonType");
        // 症状
        if (featureType.contains("1")) {
            if (map.get(LexiconEnum.Symptom.getKey()) != null) {
                List<String> list = map.get(LexiconEnum.Symptom.getKey()).stream().map(r -> r.getSonName()).collect(Collectors.toList());
                CoreUtil.removeRepeat(list, filterMap.get(StandConvertEnum.symptom.toString())); // 过滤界面已有
                pushDTO.setSymptom(generatePushBaseDTO(list, length)); // 放入对象返回
            }
        }
        // 查体结果
        if (featureType.contains("4")) {
            if (map.get(LexiconEnum.VitalResult.getKey()) != null) {
                List<String> list = map.get(LexiconEnum.VitalResult.getKey()).stream().map(r -> r.getSonName()).collect(Collectors.toList());
                CoreUtil.removeRepeat(list, filterMap.get(StandConvertEnum.vital.toString())); // 过滤界面已有
                pushDTO.setVital(generatePushBaseDTO(list, length)); // 放入对象返回
            }
        }
        // 化验套餐
        if (featureType.contains("5")) {
            if (map.get(LexiconEnum.LisName.getKey()) != null) {
                List<String> list = map.get(LexiconEnum.LisName.getKey()).stream().map(r -> r.getSonName()).collect(Collectors.toList());
                CoreUtil.removeRepeat(list, filterMap.get(StandConvertEnum.lis.toString())); // 过滤界面已有
                pushDTO.setLis(generatePushBaseDTO(list, length)); // 放入对象返回
            }
        }
        // 辅检项目
        if (featureType.contains("6")) {
            if (map.get(LexiconEnum.PacsName.getKey()) != null) {
                List<String> list = map.get(LexiconEnum.PacsName.getKey()).stream().map(r -> r.getSonName()).collect(Collectors.toList());
                CoreUtil.removeRepeat(list, filterMap.get(StandConvertEnum.pacs.toString())); // 过滤界面已有
                pushDTO.setPacs(generatePushBaseDTO(list, length)); // 放入对象返回
            }
        }
        // 鉴别诊断, 注意这个类型与其他类型不一样
        if (featureType.contains("7")) {
            if (map.get(LexiconEnum.Disease.getKey()) != null) {
                List<String> list = map.get(LexiconEnum.Disease.getKey()).stream().map(r -> r.getSonName()).collect(Collectors.toList());
                CoreUtil.removeRepeat(list, filterMap.get(StandConvertEnum.disease.toString())); // 过滤界面已有
                List<PushBaseDTO> pushBaseDTO = generatePushBaseDTO(list, length);
                if (ListUtil.isNotEmpty(pushBaseDTO)) {
                    dis.put(DiseaseTypeEnum.identify.getName(), pushBaseDTO);
                }
            }
        }
        // 药品
        if (featureType.contains("8")) {
            if (map.get(LexiconEnum.Medicine.getKey()) != null) {
                List<String> list = map.get(LexiconEnum.Medicine.getKey()).stream().map(r -> r.getSonName()).collect(Collectors.toList());
                CoreUtil.removeRepeat(list, filterMap.get(StandConvertEnum.drug.toString())); // 过滤界面已有
                pushDTO.setMedicines(generatePushBaseDTO(list, length)); // 放入对象返回
            }
        }
        // 手术
        if (featureType.contains("9")) {
            if (map.get(LexiconEnum.Operation.getKey()) != null) {
                List<String> list = map.get(LexiconEnum.Operation.getKey()).stream().map(r -> r.getSonName()).collect(Collectors.toList());
                CoreUtil.removeRepeat(list, filterMap.get(StandConvertEnum.operation.toString())); // 过滤界面已有
                pushDTO.setOperations(generatePushBaseDTO(list, length)); // 放入对象返回
            }
        }
        // 量表
        if (featureType.contains("12")) {
            if (map.get(LexiconEnum.Scale.getKey()) != null) {
                List<String> list = map.get(LexiconEnum.Scale.getKey()).stream().map(r -> r.getSonName()).collect(Collectors.toList());
                // CoreUtil.removeRepeat(list, filterMap.get(StandConvertEnum.operation.toString())); // 过滤界面已有
                pushDTO.setScale(generatePushBaseDTO(list, length)); // 放入对象返回
            }
        }
        // 护理
        if (featureType.contains("13")) {
            if (map.get(LexiconEnum.Nurse.getKey()) != null) {
                List<String> list = map.get(LexiconEnum.Nurse.getKey()).stream().map(r -> r.getSonName()).collect(Collectors.toList());
                // CoreUtil.removeRepeat(list, filterMap.get(StandConvertEnum.operation.toString())); // 过滤界面已有
                pushDTO.setNurse(generatePushBaseDTO(list, length)); // 放入对象返回
            }
        }
    }

    /**
     * 将名称列表生成推送返回类型
     *
     * @param nameList
     * @param length   最长个数, -1:无限制
     * @return
     */
    public List<PushBaseDTO> generatePushBaseDTO(List<String> nameList, int length) {
        List<PushBaseDTO> pushBaseDTOList = Lists.newArrayList();
        if (ListUtil.isNotEmpty(nameList)) {
            for (int i = 0; i < nameList.size(); i++) {
                 if (length != -1 && pushBaseDTOList.size() >= length) {
                    break;
                }
                PushBaseDTO pushBaseDTO = new PushBaseDTO();
                pushBaseDTO.setName(nameList.get(i));
                pushBaseDTOList.add(pushBaseDTO);
            }
        }
        return pushBaseDTOList;
    }

    /**
     * 生成反推的入参
     *
     * @param wordCrfDTO
     * @param featureType
     * @param dis
     * @return
     */
    public ReverseVO generateReverseVO(WordCrfDTO wordCrfDTO, List<String> featureType, Map<String, List<PushBaseDTO>> dis, boolean flag) {
        ReverseVO reverseVO = new ReverseVO();
        String disName = "";
        // 按以下顺序获取一个诊断: 下的诊断->可能诊断第一个
        if (ListUtil.isNotEmpty(wordCrfDTO.getDiag())) {
            disName = wordCrfDTO.getDiag().get(0).getUniqueName();
        } else if (dis != null) {
            List<PushBaseDTO> possibleDis = dis.get(DiseaseTypeEnum.possibleDis.getName());
            if (ListUtil.isNotEmpty(possibleDis)) {
                disName = possibleDis.get(0).getName(); // 获取可能诊断第一个
            }
        }
        reverseVO.setLibName(disName);
        // 诊断名称不为空才反推
        if (StringUtil.isNotBlank(disName)) {
            reverseVO.setAge(wordCrfDTO.getAgeNum());
            reverseVO.setSexType(wordCrfDTO.getSex());
            reverseVO.setLibType(LexiconEnum.Disease.getKey());
            // 关联类型
            List<Integer> relationType = new ArrayList<>();
            if (featureType.contains("1")) {
                relationType.add(RelationshipEnum.R501.getKey()); // 主症状
                relationType.add(RelationshipEnum.R502.getKey()); // 次症状
            }
            if (featureType.contains("4")) {
                relationType.add(RelationshipEnum.R503.getKey()); // 查体结果
            }
            if (featureType.contains("5")) {
                relationType.add(RelationshipEnum.R504.getKey()); // 化验套餐
            }
            if (featureType.contains("6")) {
                relationType.add(RelationshipEnum.R505.getKey()); // 辅助检查项目
            }
            if (featureType.contains("7")) {
                relationType.add(RelationshipEnum.R508.getKey()); // 鉴别诊断
            }
            if (flag) {
                if (featureType.contains("8")) {
                    relationType.add(RelationshipEnum.R506.getKey()); // 药品
                }
                if (featureType.contains("9")) {
                    relationType.add(RelationshipEnum.R507.getKey()); // 手术
                }
                if (featureType.contains("13")) {
                    relationType.add(RelationshipEnum.R510.getKey()); // 护理
                }
            }
            if (featureType.contains("12")) {
                relationType.add(RelationshipEnum.R509.getKey()); // 量表
            }
            reverseVO.setRelationType(relationType);
        }
        return reverseVO;
    }

    /**
     * 是否有界面诊断
     *
     * @param wordCrfDTO
     * @return
     */
    public Boolean hasDisease(WordCrfDTO wordCrfDTO) {
        if (ListUtil.isNotEmpty(wordCrfDTO.getDiag())) {
            return true;
        }
        return false;
    }

    private void reversePushPackage(int length, PushDTO pushDTO, List<String> ruleTypeList, Map<String, List<String>> typeWords, NeoPushVO pushVO, Map<String, List<PushBaseDTO>> dis) {
        NeoPushDTO reversePush = neoFacade.getReversePush(pushVO);
        // 症状
        if (ruleTypeList.contains("1")) {
            if (ListUtil.isNotEmpty(reversePush.getSymptoms())) {
                List<String> symptoms = reversePush.getSymptoms().stream().map(x -> x.getName()).collect(Collectors.toList());
                if (symptoms.size() > 0) {
                    List<PushBaseDTO> filtervitals = getPackagePushBaseDTO(typeWords, symptoms, StandConvertEnum.symptom);
                    pushDTO.setSymptom(filtervitals.subList(0, filtervitals.size() >= length ? length : filtervitals.size()));
                }
            }
        }
        // 查体
        if (ruleTypeList.contains("4")) {
            if (ListUtil.isNotEmpty(reversePush.getVitals())) {
                List<String> vitals = reversePush.getVitals().stream().map(x -> x.getName()).collect(Collectors.toList());
                if (vitals.size() > 0) {
                    List<PushBaseDTO> filtervitals = getPackagePushBaseDTO(typeWords, vitals, StandConvertEnum.vital);
                    pushDTO.setVital(filtervitals.subList(0, filtervitals.size() >= length ? length : filtervitals.size()));
                }
            }
        }
        // 化验
        if (ruleTypeList.contains("5")) {
            if (ListUtil.isNotEmpty(reversePush.getLis())) {
                List<String> lises = reversePush.getLis().stream().map(x -> x.getName()).collect(Collectors.toList());
                if (lises.size() > 0) {
                    List<PushBaseDTO> filterlis = getPackagePushBaseDTO(typeWords, lises, StandConvertEnum.lis);
                    pushDTO.setLis(filterlis.subList(0, filterlis.size() >= length ? length : filterlis.size()));
                }
            }
        }
        // 辅检
        if (ruleTypeList.contains("6")) {
            if (ListUtil.isNotEmpty(reversePush.getPacs())) {
                List<String> pacses = reversePush.getPacs().stream().map(x -> x.getName()).collect(Collectors.toList());
                if (pacses.size() > 0) {
                    List<PushBaseDTO> filterpacs = getPackagePushBaseDTO(typeWords, pacses, StandConvertEnum.pacs);
                    pushDTO.setPacs(filterpacs.subList(0, filterpacs.size() >= length ? length : filterpacs.size()));
                }
            }
        }
        // 药品
        if (ruleTypeList.contains("8")) {
            if (reversePush.getTreat() != null && ListUtil.isNotEmpty(reversePush.getTreat().get(MedicalAdviceEnum.drug.getName()))) {
                List<String> drugs = reversePush.getTreat().get(MedicalAdviceEnum.drug.getName()).stream().map(x -> x.getName()).collect(Collectors.toList());
                /*reversePush.stream().filter(x ->x.getTreat() != null && ListUtil.isNotEmpty(x.getTreat().get(MedicalAdviceEnum.drug.getName())))
                        .forEach(x -> {
                            List<String> drugList = x.getTreat().get(MedicalAdviceEnum.drug.getName()).stream().map(y -> y.getName()).collect(Collectors.toList());
                            drugs.removeAll(drugList);
                            drugs.addAll(drugList);
                        });*/
                if (drugs.size() > 0) {
                    List<PushBaseDTO> filterdrugs = getPackagePushBaseDTO(typeWords, drugs, StandConvertEnum.drug);
                    pushDTO.setMedicines(filterdrugs.subList(0, filterdrugs.size() >= length ? length : filterdrugs.size()));
                }
            }
        }
        // 手术
        if (ruleTypeList.contains("9")) {
            if (reversePush.getTreat() != null && ListUtil.isNotEmpty(reversePush.getTreat().get(MedicalAdviceEnum.operation.getName()))) {

                List<String> operations = reversePush.getTreat().get(MedicalAdviceEnum.operation.getName()).stream().map(x -> x.getName()).collect(Collectors.toList());
                if (operations.size() > 0) {
                    List<PushBaseDTO> filteroperations = getPackagePushBaseDTO(typeWords, operations, StandConvertEnum.operation);
                    pushDTO.setOperations(filteroperations.subList(0, filteroperations.size() >= length ? length : filteroperations.size()));
                }
            }
        }
        //反推，推鉴别诊断
        if (ruleTypeList.contains("7")) {
            if (ListUtil.isNotEmpty(reversePush.getDifferentialDis())) {
                List<String> differentDis = reversePush.getDifferentialDis().stream().map(x -> x.getName()).collect(Collectors.toList());
                if (differentDis.size() > 0) {
                    List<PushBaseDTO> filterpacs = getPackagePushBaseDTO(typeWords, differentDis, StandConvertEnum.disease);
                    List<PushBaseDTO> pushDiffBaseDTOS = filterpacs.subList(0, filterpacs.size() >= length ? length : filterpacs.size());
                    dis.put(DiseaseTypeEnum.identify.getName(), pushDiffBaseDTOS);
                    pushDTO.setDis(dis);
                }
            }
        }
        // 量表
        if (ruleTypeList.contains("12")) {
            if (ListUtil.isNotEmpty(reversePush.getScales())) {
                List<String> scales = reversePush.getScales().stream().map(x -> x.getName()).collect(Collectors.toList());
                if (scales.size() > 0) {
                    List<PushBaseDTO> filterpacs = getPackagePushBaseDTO(scales);
                    pushDTO.setScale(filterpacs.subList(0, filterpacs.size() >= length ? length : filterpacs.size()));
                }
            }
        }

    }

    private List<PushBaseDTO> getPackagePushBaseDTO(Map<String, List<String>> typeWords, List<String> symptoms, StandConvertEnum symptom) {
        List<PushBaseDTO> symptomPush = symptoms.stream()
                .map(x -> {
                    x = ListUtil.isNotEmpty(typeWords.get(symptom.toString())) ? !typeWords.get(symptom.toString()).contains(x) ? x : "" : x;
                    return x;
                }).filter(x -> StringUtils.isNotBlank(x)).map(x -> {
                    PushBaseDTO pushBaseDTO = new PushBaseDTO();
                    pushBaseDTO.setName(x);
                    return pushBaseDTO;
                }).collect(Collectors.toList());
        return symptomPush;
    }
    private List<PushBaseDTO> getPackagePushBaseDTO(List<String> symptoms) {
        List<PushBaseDTO> symptomPush = new ArrayList<>();
        if(ListUtil.isNotEmpty(symptoms)){
            symptomPush = symptoms.stream().map(x ->{
                PushBaseDTO pushBaseDTO = new PushBaseDTO();
                pushBaseDTO.setName(x);
                return pushBaseDTO;
            }).collect(Collectors.toList());
        }

        return symptomPush;
    }
}

