package com.diagbot.rule;

import com.diagbot.dto.BillMsg;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.BaseTypeEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MsgUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @description: 辅检规则
 * @author: kwz
 * @time: 2020/8/3 14:47
 */
@Component
public class PacsRule {

    @Autowired
    AgeRule ageRule;
    @Autowired
    SexRule sexRule;
    @Autowired
    GroupRule groupRule;
    @Autowired
    CommonRule commonRule;

    /**
     * 比较辅检名称——开单合理性
     *
     * @param inputList
     * @param ruleBaseDTO
     * @param billMsgList
     * @param conType
     * @param ruleSimpleDTO
     * @param <T>
     */
    public <T> void bill(List<T> inputList, RuleBaseDTO ruleBaseDTO, List<BillMsg> billMsgList, String conType,
                         RuleSimpleDTO ruleSimpleDTO) {
        if (ListUtil.isNotEmpty(inputList)) {
            for (T lis : inputList) {
                String val = (String) CoreUtil.getFieldValue(lis, "uniqueName");
                String val_name = (String) CoreUtil.getFieldValue(lis, "name");
                if (StringUtils.isNotBlank(val) && CoreUtil.compareName(ruleBaseDTO, val)) {
                    BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(
                            ruleSimpleDTO.getInputName(), ruleSimpleDTO.getLibName(),
                            "已开" + val_name, conType, ruleSimpleDTO.getLibTypeName());
                    billMsgList.add(commonBillMsg);
                }
            }
        }
    }

    /**
     * 比较辅检——危急值
     *
     * @param contentList
     * @param criticalList
     */
    public void critical(List<String> contentList, List<BillMsg> criticalList) {
        for (String s : contentList) {
            BillMsg commonBillMsg = MsgUtil.getCommonCriticalMsg(s, TypeEnum.pacs.getName());
            criticalList.add(commonBillMsg);
        }
    }

    /**
     * 比较辅检结果——其他提醒
     *
     * @param ruleBaseDTOList
     * @param msg
     * @param otherList
     * @param wordCrfDTO
     */
    public void otherPacs(List<RuleBaseDTO> ruleBaseDTOList, String msg, List<BillMsg> otherList, WordCrfDTO wordCrfDTO) {
        if (ListUtil.isEmpty(ruleBaseDTOList)) {
            return ;
        }
        int i = 0;
        for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
            if (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType()) != null) { // 避免空指针
                boolean flag = false;
                switch (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType())) {
                    case Age: // 年龄
                        flag = ageRule.getFlag(wordCrfDTO, ruleBaseDTO);
                        if (flag) {
                            i++;
                        }
                        break;
                    case Gender: // 性别
                        flag = sexRule.getFlag(wordCrfDTO, ruleBaseDTO);
                        if (flag) {
                            i++;
                        }
                        break;
                    case Group: // 人群
                        Map map = groupRule.rule(wordCrfDTO, ruleBaseDTO);
                        if (CoreUtil.getMapFlag(map)) {
                            i++;
                        }
                        break;
                    case PacsResult: // 辅检结果
                        map = commonRule.compareItem(wordCrfDTO.getPacsLabel().getRes(), ruleBaseDTO);
                        if (ruleBaseDTO.getBaseType().equals(BaseTypeEnum.B3.getKey())) {
                            if (!CoreUtil.getMapFlag(map)) {
                                i++;
                            }
                        } else {
                            if (CoreUtil.getMapFlag(map)) {
                                i++;
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        // 条件全部符合
        if (i == ruleBaseDTOList.size()) {
            BillMsg otherMsg = MsgUtil.getCommonOtherMsg(TypeEnum.pacs.getName(), msg, msg, null, null);
            otherList.add(otherMsg);
        }
    }
}
