package com.diagbot.rule;

import com.diagbot.dto.BillMsg;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.model.label.VitalLabel;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.MsgUtil;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @description: 体征规则
 * @author: kwz
 * @time: 2020/8/3 14:47
 */
@Component
public class VitalRule {

    /**
     * 比较体征——开单合理性
     *
     * @param vitalLabel
     * @param ruleBaseDTO
     * @param billMsgList
     * @param conType
     * @param ruleSimpleDTO
     */
    public void bill(VitalLabel vitalLabel, RuleBaseDTO ruleBaseDTO, List<BillMsg> billMsgList, String conType,
                     RuleSimpleDTO ruleSimpleDTO) {
        Map<String, Object> map = CoreUtil.compareVital(ruleBaseDTO, vitalLabel);
        List<String> msgList = CoreUtil.getMapMsgList(map);
        for (String s : msgList) {
            BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(ruleSimpleDTO.getInputName(),
                    ruleSimpleDTO.getLibName(), s, conType, ruleSimpleDTO.getLibTypeName());
            billMsgList.add(commonBillMsg);
        }
    }

    /**
     * 比较体征——推送
     *
     * @param vitalLabel
     * @param ruleBaseDTO
     * @param baseIdList
     * @param ids
     */
    public void push(VitalLabel vitalLabel, RuleBaseDTO ruleBaseDTO, List<Long> baseIdList, String ids) {
        Map<String, Object> map = CoreUtil.compareVital(ruleBaseDTO, vitalLabel);
        if (CoreUtil.getMapFlag(map)) {
            CoreUtil.addSplitString(baseIdList, ids);
        }
    }

    /**
     * 比较体征——高危手术
     *
     * @param vitalLabel
     * @param ruleBaseDTO
     * @param highRiskList
     * @param ruleSimpleDTO
     */
    public void highOperation(VitalLabel vitalLabel, RuleBaseDTO ruleBaseDTO, List<BillMsg> highRiskList, RuleSimpleDTO ruleSimpleDTO) {
        Map<String, Object> map = CoreUtil.compareVital(ruleBaseDTO, vitalLabel);
        List<String> msgList = CoreUtil.getMapMsgList(map);
        for (String s : msgList) {
            BillMsg billMsg = MsgUtil.getCommonHighRiskMsg(s, ruleSimpleDTO.getInputName(), ruleSimpleDTO.getLibTypeName());
            highRiskList.add(billMsg);
        }
    }
}
