package com.diagbot.vo;

import cn.afterturn.easypoi.excel.annotation.Excel;
import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Pacs;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 测试开单合理性
 * </p>
 *
 * @author zhaops
 * @since 2020-07-28
 */
@Data
public class TestIndicationVO implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 规则类型（1:危急值提醒，2：开单合理项，3：管理评估，4：不良反应，5：药物推荐，6：异常值）
     */
    @NotBlank(message = "ruleType不能为空")
    private String ruleType = "";

    /**
     * 大数据返回内容截取长度
     */
    private Integer length = 10;
    /**
     * 年龄
     */
    @Excel(name="年龄")
    private Double age;
    /**
     * 性别（1：男，2：女，3：通用）
     */
    private Integer sex;
    /**
     * 婚姻
     */
    private String marriage = "";
    /**
     * 主诉
     */
    @Excel(name="主诉")
    private String chief = "";
    /**
     * 现病史
     */
    @Excel(name="现病史")
    private String symptom = "";
    /**
     * 查体
     */
    private String vital = "";
    /**
     * 既往史
     */
    @Excel(name="既往史")
    private String pasts = "";
    /**
     * 传染病史
     */
    private String infectious = "";
    /**
     * 手术外伤史
     */
    private String operation = "";
    /**
     * 过敏史
     */
    private String allergy = "";
    /**
     * 接种史
     */
    private String vaccination = "";
    /**
     * 个人史
     */
    private String personal = "";
    /**
     * 婚育史
     */
    private String marital = "";
    /**
     * 家族史
     */
    private String family = "";
    /**
     * 月经史
     */
    private String menstrual = "";
    /**
     * 其他史
     */
    private String other = "";
    /**
     * 化验文本数据
     */
    private String lisString = "";
    /**
     * 辅检文本数据
     */
    private String pacsString = "";
    /**
     * 诊断文本数据
     */
    private String diagString = "";
    /**
     * 药品文本数据
     */
    private String drugString = "";
    /**
     * 科室
     */
    private List<Item> dept = new ArrayList<>();
    /**
     * 化验项目和结果
     */
    private List<Lis> lis = new ArrayList<>();
    /**
     * 辅检项目和结果
     */
    private List<Pacs> pacs = new ArrayList<>();
    /**
     * 诊断
     */
    private List<Item> diag = new ArrayList<>();
    /**
     * 药品
     */
    private List<Item> drug = new ArrayList<>();
    /**
     * 当前化验开单项
     */
    private List<Lis> lisOrder = new ArrayList<>();
    /**
     * 当前辅检开单项
     */
    private List<Pacs> pacsOrder = new ArrayList<>();
    /**
     * 当前诊断开单项
     */
    private List<Item> diagOrder = new ArrayList<>();
    /**
     * 当前药品开单项
     */
    private List<Item> drugOrder = new ArrayList<>();
    /**
     * 当前手术开单项
     */
    private List<Item> operationOrder = new ArrayList<>();
    /**
     * 其他开单项
     */
    private List<Item> otherOrder = new ArrayList<>();
    /**
     * 选中诊断
     */
    private Item diseaseName;



    /************************************************扩展字段公共部分********************************/
    @Excel(name="序号")
    private String idNum;
    @Excel(name="医保手术和操作名称")
    private String operationName;
    @Excel(name="开单项类型")
    private String orderType;
    @Excel(name="标准名称")
    private String standName;
    @Excel(name="内容标准名称")
    private String neoName;
    @Excel(name="禁忌类型")
    private String neoType;
    @Excel(name="输血治疗")
    private String transfusionName;
    @Excel(name="节点标签")
    private String transfusionNodeName;
    @Excel(name="实验室检查套餐")
    private String criticalMeal;
    @Excel(name="低危急值")
    private Double criticalMin;
    @Excel(name="高危急值")
    private Double criticalMax;
    @Excel(name="参考对象")
    private String criticalAge;
    @Excel(name="手术级别")
    private String operationLevel;
    @Excel(name="药品高危级别")
    private String drugLevel;
    @Excel(name="药品通用名称")
    private String drugHighRisk;
    @Excel(name="注册剂型")
    private String drugForm;
    @Excel(name="实验室检查名称")
    private String lisGBName;
    @Excel(name="比较符号")
    private String symbol;
    @Excel(name="数值")
    private Double value;
    @Excel(name="标签")
    private String otherTipNodeType;
    @Excel(name="名称")
    private String otherTipNodeName;
    @Excel(name="医保疾病名称")
    private String disName;
    @Excel(name="指标名称")
    private String itemName;
    @Excel(name="指标标签")
    private String itemNodeName;
    @Excel(name="年龄最大值")
    private Double ageMax;
    @Excel(name="年龄最小值")
    private Double ageMin;
    @Excel(name="年龄范围")
    private Double ageRange;
    @Excel(name="检查结果")
    private String pacsDesc;
    @Excel(name="检查结论有")
    private String pacsDescHas;
    @Excel(name="性别")
    private String sexStr;

    @Excel(name="生命体征及查体")
    private String physique;
    @Excel(name="合并疾病")
    private String disease;
    @Excel(name="化验结果")
    private String lisAll;
    @Excel(name="辅检结果描述")
    private String pacsAll;
    @Excel(name="年龄T")
    private String ageAll;
    @Excel(name="规则")
    private String ruleText;
    @Excel(name="药品类型")
    private String drugType;

    @Excel(name="状态")
    private String status;
    @Excel(name="辅助检查名称")
    private String pacsOrderName;
    @Excel(name="正常规则")
    private String ruleContent;
}
