package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.DiagnosesFindDTO;
import com.diagbot.dto.GetAllForRelationDTO;
import com.diagbot.dto.KlDiagnoseInfoDTO;
import com.diagbot.dto.KlDiagnoseTypeDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.enums.StatusEnum;
import com.diagbot.facade.KlDiagnoseFacade;
import com.diagbot.vo.DiagnosesFindVO;
import com.diagbot.vo.KlDiagnoseByIdVO;
import com.diagbot.vo.KlDiagnoseClearVO;
import com.diagbot.vo.KlDiagnoseInfoVO;
import com.diagbot.vo.KlDiagnoseSatarDisVO;
import com.diagbot.vo.KlDiagnoseSaveVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * @author wangfeng
 * @Description:
 * @date 2021-03-19 14:52
 */
@RestController
@RequestMapping("/klDiagnose")
@Api(value = "诊断依据维护相关API", tags = { "诊断依据维护相关API" })
@SuppressWarnings("unchecked")
public class KlDiagnoseController {
    @Autowired
    KlDiagnoseFacade klDiagnoseFacade;

    @ApiOperation(value = "分页获取诊断依据维护列表[by:wangfeng]")
    @PostMapping("/getKlDiagnosePage")
    @SysLogger("getKlDiagnosePage")
    public RespDTO<IPage<KlDiagnoseInfoDTO>> getDiagnosePages(@RequestBody KlDiagnoseInfoVO klRuleInfoVO) {
        return RespDTO.onSuc(klDiagnoseFacade.getKlDiagnoseInfoPage(klRuleInfoVO));
    }

    @ApiOperation(value = "根据Id获取诊断依据详情[by:wangfeng]")
    @PostMapping("/getByIdDiagnose")
    @SysLogger("getByIdDiagnose")
    public RespDTO<List<KlDiagnoseTypeDTO>> getByIdDiagnoseAll(@RequestBody @Valid KlDiagnoseByIdVO klDiagnoseByIdVO) {
        return RespDTO.onSuc(klDiagnoseFacade.getByIdDiagnoseAll(klDiagnoseByIdVO));
    }

    @ApiOperation(value = "保存诊断依据详情[by:wangfeng]")
    @PostMapping("/saveDiagnose")
    @SysLogger("saveDiagnose")
    @Transactional
    public RespDTO<Boolean> saveDiagnoseAll(@RequestBody @Valid KlDiagnoseSaveVO klDiagnoseSaveVO) {
        return RespDTO.onSuc(klDiagnoseFacade.saveDiagnoseAll(klDiagnoseSaveVO));
    }

    @ApiOperation(value = "刪除诊断依据详情[by:wangfeng]")
    @PostMapping("/clearDiagnose")
    @SysLogger("clearDiagnose")
    @Transactional
    public RespDTO<Boolean> clearDiagnoseAll(@RequestBody @Valid KlDiagnoseClearVO klDiagnoseClearVO) {
        return RespDTO.onSuc(klDiagnoseFacade.clearDiagnoseAll(klDiagnoseClearVO));
    }

    @ApiOperation(value = "停用诊断依据[by:wangfeng]")
    @PostMapping("/disableKlDiagnose")
    @SysLogger("disableKlDiagnose")
    public RespDTO<Boolean> disableDiagnoses(@RequestBody @Valid KlDiagnoseSatarDisVO klDiagnoseSatarDisVO) {
        return RespDTO.onSuc(klDiagnoseFacade.disableDiagnose(klDiagnoseSatarDisVO, StatusEnum.Disable.getKey()));
    }

    @ApiOperation(value = "启用诊断依据[by:wangfeng]")
    @PostMapping("/startKlDiagnose")
    @SysLogger("startKlDiagnose")
    public RespDTO<Boolean> startKlDiagnoses(@RequestBody @Valid KlDiagnoseSatarDisVO klDiagnoseSatarDisVO) {
        return RespDTO.onSuc(klDiagnoseFacade.disableDiagnose(klDiagnoseSatarDisVO, StatusEnum.Enable.getKey()));
    }

    @ApiOperation(value = "搜索未添加的诊断依据术语名[by:wangfeng]",
            notes = "diseaseName：疾病医学术语名称 <br>")
    @PostMapping("/findDiaName")
    @SysLogger("findDiaName")
    public RespDTO<List<DiagnosesFindDTO>> findDiaNames(@Valid @RequestBody DiagnosesFindVO diagnosesFindVO) {
        List<DiagnosesFindDTO> klConcepts = klDiagnoseFacade.findDiaNameAlls(diagnosesFindVO);
        return RespDTO.onSuc(klConcepts);
    }
}
