package com.diagbot.web;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.RegularValueDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.KlRegularConfig;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.facade.KlRegularConfigFacade;
import com.diagbot.vo.RegularConfigDataVO;
import com.diagbot.vo.RegularConfigSaveVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zhoutg
 * @since 2020-11-30
 */
@RestController
@RequestMapping("/sysRegularConfig")
@Api(value = "正则匹配相关API", tags = { "正则匹配相关API" })
@SuppressWarnings("unchecked")
public class KlRegularConfigController {

    @Autowired
    KlRegularConfigFacade klRegularConfigFacade;


    @ApiOperation(value = "从文本中取出化验相应正则数据",
            notes = "从文本中取出化验相应正则数据")
    @PostMapping("/getRegularConfigData")
    public RespDTO<List<RegularValueDTO>> getRegularConfigData(@Valid @RequestBody RegularConfigDataVO regularConfigDataVO) {
        // 现病史化验和过敏史
        QueryWrapper<KlRegularConfig> regularQuery = new QueryWrapper<>();
        regularQuery.eq("is_deleted", IsDeleteEnum.N.getKey())
        .eq(regularConfigDataVO.getRulesCode() != null, "rules_code", regularConfigDataVO.getRulesCode())
                .eq(regularConfigDataVO.getRulesTepy() != null, "rules_tepy", regularConfigDataVO.getRulesTepy());
        List<KlRegularConfig> regularConfigs = klRegularConfigFacade.list(regularQuery);
        List<RegularValueDTO> data = klRegularConfigFacade.getRegularConfigDatas(regularConfigDataVO.getText(), regularConfigs);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "保存相应正则数据",
            notes = "保存相应正则数据")
    @PostMapping("/saverRegularConfigData")
    @Transactional
    public RespDTO<Boolean> saverRegularConfigData(@Valid @RequestBody List<RegularConfigSaveVO> regularConfigSaveVO) {
        boolean res = klRegularConfigFacade.saverRegularConfigDatas(regularConfigSaveVO);
        return RespDTO.onSuc(res);
    }

}
