package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.KlRuleByIdParDTO;
import com.diagbot.dto.KlRuleInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.enums.StatusEnum;
import com.diagbot.facade.KlRuleFacade;
import com.diagbot.vo.KlRuleByIdVO;
import com.diagbot.vo.KlRuleInfoClearVO;
import com.diagbot.vo.KlRuleInfoSaveVO;
import com.diagbot.vo.KlRuleInfoVO;
import com.diagbot.vo.KlRuleSatartOrdisaVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * <p>
 * 规则信息表 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2021-03-05
 */
@RestController
@RequestMapping("/klRule")
@Api(value = "规则维护相关API", tags = { "规则维护相关API" })
@SuppressWarnings("unchecked")
public class KlRuleController {

    @Autowired
    KlRuleFacade klRuleFacade;

    @ApiOperation(value = "分页获取规则维护列表[by:wangfeng]")
    @PostMapping("/getKlRuleInfoPage")
    @SysLogger("getKlRuleInfoPage")
    public RespDTO<IPage<KlRuleInfoDTO>> getKlRuleInfoPages(@RequestBody KlRuleInfoVO klRuleInfoVO) {
        return RespDTO.onSuc(klRuleFacade.getKlRuleInfoPage(klRuleInfoVO));
    }

    @ApiOperation(value = "根据规则Id获取规则详情[by:wangfeng]")
    @PostMapping("/getByIdRuleInfo")
    @SysLogger("getByIdRuleInfo")
    public RespDTO<KlRuleByIdParDTO> getByIdRuleInfoAll(@RequestBody @Valid KlRuleByIdVO klRuleByIdVO) {
        return RespDTO.onSuc(klRuleFacade.getByIdRuleInfos(klRuleByIdVO));
    }

    @ApiOperation(value = "保存规则详情[by:wangfeng]")
    @PostMapping("/saveRuleInfo")
    @SysLogger("saveRuleInfo")
    @Transactional
    public RespDTO<Boolean> saveRuleInfoAll(@RequestBody @Valid KlRuleInfoSaveVO klRuleInfoSaveVO) {
        return RespDTO.onSuc(klRuleFacade.saveRuleInfos(klRuleInfoSaveVO));
    }

    @ApiOperation(value = "刪除规则详情[by:wangfeng]")
    @PostMapping("/clearRuleInfo")
    @SysLogger("clearRuleInfo")
    @Transactional
    public RespDTO<Boolean> clearRuleInfoAll(@RequestBody @Valid KlRuleInfoClearVO klRuleInfoClearVO) {
        return RespDTO.onSuc(klRuleFacade.clearRuleInfos(klRuleInfoClearVO));
    }

    @ApiOperation(value = "停用规则[by:wangfeng]")
    @PostMapping("/disableRuleInfo")
    @SysLogger("disableRuleInfo")
    public RespDTO<Boolean> disableRuleInfos(@RequestBody @Valid KlRuleSatartOrdisaVO klRuleSatartOrdisaVO) {
        return RespDTO.onSuc(klRuleFacade.startOrDisableRule(klRuleSatartOrdisaVO,StatusEnum.Disable.getKey()));
    }

    @ApiOperation(value = "启用规则[by:wangfeng]")
    @PostMapping("/startRuleInfo")
    @SysLogger("startRuleInfo")
    public RespDTO<Boolean> startRuleInfos(@RequestBody @Valid KlRuleSatartOrdisaVO klRuleSatartOrdisaVO) {
        return RespDTO.onSuc(klRuleFacade.startOrDisableRule(klRuleSatartOrdisaVO,StatusEnum.Enable.getKey()));
    }

}
