package com.diagbot.web;

import com.diagbot.dto.IndexBatchDTO;
import com.diagbot.dto.KllisDetailDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.facade.KlConceptFacade;
import com.diagbot.facade.MedRetrievalFacade;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.KllisDetailVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/12 16:45
 */
@RestController
@RequestMapping("/retrieval")
@Api(value = "术语检索相关API", tags = { "术语检索相关API" })
@SuppressWarnings("unchecked")
public class RetrievalController {
    @Autowired
    private MedRetrievalFacade medRetrievalFacade;
    @Autowired
    private KlConceptFacade klConceptFacade;


    @ApiOperation(value = "术语检索[zhaops]",
            notes = "type： 类型：1-化验大项、2-化验小项、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、9-症状、10-量表、11-护理、12-中医诊断、13-中医证候、14-麻醉 <br>" +
                    "inputStr： 检索内容<br>" +
                    "sex： 性别：1-男、2-女、3-通用 <br>" +
                    "age： 年龄<br>")
    @PostMapping("/index")
    public RespDTO<RetrievalDTO> index(@Valid @RequestBody RetrievalVO retrievalVO) {
        RetrievalDTO data = medRetrievalFacade.index(retrievalVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "医学知识（静态知识）检索[zhaops]",
            notes = "types： 类型（多选）：0-全部、1-诊断、2-药品、3-检验、5-检查、6-手术和操作、8-量表 <br>" +
                    "inputStr： 检索内容<br>")
    @PostMapping("/staticKnowledgeIndex")
    public RespDTO<List<StaticKnowledgeIndexDTO>> staticKnowledgeIndex(@Valid @RequestBody StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<StaticKnowledgeIndexDTO> data = medRetrievalFacade.staticKnowledgeIndex(staticKnowledgeIndexVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "术语批量查询[zhaops]",
            notes = "type： 类型：1-化验大项、2-化验小项、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候<br>" +
                    "ids： 术语id列表<br>" +
                    "names： 术语列表<br>" +
                    "source： 数据来源：-1-id匹配、1-标准词、2-同义词、3-编码、4-历史数据、5-相似词、99-数据迁移<br>")
    @PostMapping("/getConceptNames")
    public RespDTO<List<IndexBatchDTO>> getConceptNames(@Valid @RequestBody ConceptVO conceptVO) {
        List<IndexBatchDTO> data = klConceptFacade.getConceptNames(conceptVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "化验小项术语批量查询获取范围[gaodm]",
            notes = "names： 术语列表<br>")
    @PostMapping("/getLisDetaisByNames")
    public RespDTO<Map<String, KllisDetailDTO>> getLisDetaisByNames(@RequestBody KllisDetailVO kllisDetailVO) {
        return RespDTO.onSuc(klConceptFacade.getLisDetaisByNamesFac(kllisDetailVO));
    }
}